import globalPluginHandler
from scriptHandler import script
from logHandler import log
import ui
import speech
import os
import urllib
import urllib.request
import urllib.parse
from urllib.request import urlopen
import locale
import wx
import wx.adv
import wx.lib.mixins.listctrl as listmix
from .xml.etree import ElementTree as ET
import http.cookiejar
import re
import threading
import time
import gui
from gui import guiHelper
from os.path import expanduser
from . import csv
import datetime
import winsound
import tones
from NVDAObjects.IAccessible import IAccessible, getNVDAObjectFromEvent
import api
import controlTypes
import webbrowser
import json
import addonHandler
from .bs4 import BeautifulSoup
import requests
from speech import sayAll
from collections import defaultdict
try:
  from . import vlc
except:
  vlcInstalled = False
else:
  vlcInstalled = True
import math
from . import podsearch
import uuid

locale.setlocale(locale.LC_ALL, "czech")
addonHandler.initTranslation()

RssFolder = os.path.join(os.getenv('APPDATA'), "Marsus", "NVDA", "Rss")
articlesFolder = os.path.join(RssFolder, "Articles")
archiveFolder = os.path.join(RssFolder, "Archive")
opmlFilePath = os.path.join(RssFolder, "channels.opml")
settingsFilePath = os.path.join(RssFolder, "settings.json")
articlesHeader = ['title', 'description', 'date', 'link', 'deleted', 'audiourl', 'audiopos', 'uid']
# Translators: Application title
mainTitle = _("RSS reader")
soundPath = os.path.join(os.path.dirname(__file__), "sound")
currentArticleDescription = ""
currentArticlePosition = ""
currentChannelNewMessages = ""
currentChannelResults = ""
openArticlesAsText = False
askArticleDeletion = False
askArticleArchivation = True
automaticallyAnnounceNumberofNews = True
automaticallyAnnounceDescription = True
newArticlesAfterOpening = True
downloadArticlesAfterStartup = False
downloadArticlesInterval = 0
toTrayOnNvdaStart = False
deleteOlderArticles = 0
podcastsMp3Path = ""
saveAudioFromPodcasts = False
defaultPodcastMp3Folder = os.path.join(expanduser("~"), "Documents", "RSS podcasty")
prefixPodcastFile = 1
searchFilter = False
inAllArticles = False
deleteAllAfter = defaultdict(list)
inProcessOfReadingNewArticles = False
trayNewArticlesCount = 0
icon = wx.Icon(os.path.join(os.path.dirname(__file__), "files", "rsstrayicon.ico"), wx.BITMAP_TYPE_ICO)
channelHwnd = None
articleHwnd = None
# Translators: Message asking the user to wait as messages are being retrieved
inprocessMessage = _("Receiving messages in progress, please wait until it is complete.")

_addonDir = os.path.join(os.path.dirname(__file__), "..", "..")
_curAddon = addonHandler.Addon(_addonDir)
_addonName = _curAddon.manifest['name']
_addonSummary = _curAddon.manifest['summary']
_addonVersion = _curAddon.manifest['version']
_addonPage = _curAddon.manifest['url']

def validFileName(s):
  validName = ""
  for i in s:
    if i in "\/:*?<>|\"":
      if i == ":":
        i = " -" 
      elif i == "\"":
        i = "" 
      else:
        i = "-"
    validName = validName + i
  return validName

def playSound(fSoundName, looped=False):
  fSound = os.path.join(soundPath, fSoundName)
  if os.path.isfile(fSound):
    if looped:
      winsound.PlaySound(fSound, winsound.SND_FILENAME + winsound.SND_LOOP + winsound.SND_ASYNC)
    else:
      winsound.PlaySound(fSound, winsound.SND_FILENAME)

def stopsound():
      winsound.PlaySound(None, winsound.SND_PURGE)

def convertDownloadArticlesInterval(i):
  if i == 1: # 5 minut
    return 300000
  elif i == 2: # 10 minut
    return 600000
  elif i == 3: # 15 minut
    return 900000
  elif i == 4: # 30 minut
    return 1800000
  elif i == 5: # 1 hodina
    return 3600000
  elif i == 6: # 2 hodiny
    return 7200000
  elif i == 7: # 3 hodiny
    return 10800000
  else:
    return 0

def convertDeleteOlderArticles(i):
  if i == 1: # Jeden den
    return 1
  elif i == 2: # Dva dny
    return 2
  elif i == 3: # Tři dny
    return 3
  elif i == 4: # Pět dnů
    return 5
  elif i == 5: # Jeden týden
    return 7
  elif i == 6: # Dva týdny
    return 14
  elif i == 7: # Tři týdny
    return 21
  elif i == 8: # Jeden měsíc
    return 30
  else:
    return 0

def readcfg():
  global openArticlesAsText
  global askArticleDeletion
  global askArticleArchivation
  global automaticallyAnnounceNumberofNews
  global automaticallyAnnounceDescription
  global newArticlesAfterOpening
  global downloadArticlesAfterStartup
  global downloadArticlesInterval
  global toTrayOnNvdaStart
  global deleteOlderArticles
  global podcastsMp3Path
  global saveAudioFromPodcasts
  global prefixPodcastFile
  if not os.path.isfile(settingsFilePath):
    config = {"OpenArticlesAsText": True, "AskArticleDeletion": True, "AskArticleArchivation": True, "AutomaticallyAnnounceNumberofNews": True, "AutomaticallyAnnounceDescription": True, "NewArticlesAfterOpening": True, "DownloadArticlesAfterStartup": False, "DownloadArticlesInterval": 0, "ToTrayOnNvdaStart": False, "DeleteOlderArticles": 0, "PodcastsMp3Path": "", "SaveAudioFromPodcasts": False, "PrefixPodcastFile": 1}
    SetFile = settingsFilePath
    with open(SetFile, 'w') as f:
      json.dump(config, f)    
  SetFile = settingsFilePath
  with open(SetFile, 'r') as f:
    config = json.load(f)
  try:
    openArticlesAsText = config['OpenArticlesAsText']
  except: pass
  try:
    askArticleDeletion = config['AskArticleDeletion']
  except: pass
  try:
    askArticleArchivation = config['AskArticleArchivation']
  except: pass
  try:
    automaticallyAnnounceNumberofNews = config['AutomaticallyAnnounceNumberofNews']
  except: pass
  try:
    automaticallyAnnounceDescription = config['AutomaticallyAnnounceDescription']
  except: pass
  try:
    newArticlesAfterOpening = config['NewArticlesAfterOpening']
  except: pass
  try:
    downloadArticlesAfterStartup = config['DownloadArticlesAfterStartup']
  except: pass
  try:
    downloadArticlesInterval = config['DownloadArticlesInterval']
  except: pass
  try:
    toTrayOnNvdaStart = config['ToTrayOnNvdaStart']
  except: pass
  try:
    deleteOlderArticles = config['DeleteOlderArticles']
  except: pass
  try:
    podcastsMp3Path = config['PodcastsMp3Path']
  except: pass
  try:
    saveAudioFromPodcasts = config['SaveAudioFromPodcasts']
  except: pass
  try:
    prefixPodcastFile = config['PrefixPodcastFile']
  except: pass
  # when podcastsMp3Path is not set, assign and create default one
  global defaultPodcastMp3Folder
  if not podcastsMp3Path:
    podcastsMp3Path = defaultPodcastMp3Folder
    if not os.path.exists(podcastsMp3Path):
      os.mkdir(podcastsMp3Path)

def openinVlc(audioStream):
  import subprocess
  prgFilesPath = ""
  # find path to program files x86 folder
  if os.environ.get("PROGRAMFILES(X86)") is None:
    # in this case the system is 32bit
    prgFilesPath = os.environ.get("PROGRAMFILES")
  else:
    prgFilesPath = os.environ.get("PROGRAMFILES(X86)")
  vlcPath = os.path.join(prgFilesPath, "VideoLAN", "VLC", "vlc.exe")
  subprocess.Popen([vlcPath, audioStream])

def getRightDateFormat(articleDate):
  locale.setlocale(locale.LC_TIME, "en")
  if articleDate:
    try:
      artDateTime = datetime.datetime.strptime(articleDate, "%a, %d %b %Y %H:%M:%S %z")
    except Exception:
      try:
        # another date and time format
        artDateTime = datetime.datetime.strptime(articleDate, "%a, %d %b %Y %H:%M:%S %Z")
      except Exception:
        try:
          # date and time format in atom
          artDateTime = datetime.datetime.strptime(articleDate, "%Y-%m-%dT%H:%M:%S%z")
        except Exception:
          artDateTime = None
  else:
    artDateTime = None
  return artDateTime

# downloading with progressbar
# taken from NVDA source, slightly edited

class Downloader:

  def __init__(self, downloadUrl, downloadTarget, quiet=False, callingProcess=""):
    self.downloadUrl = downloadUrl
    self.downloadTarget = downloadTarget
    self.quiet = quiet
    self.callingProcess = callingProcess
    self.downloadThread = None

  def downloadWithoutProgress(self):
    headers = {
      'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
    }
    try:
      response = requests.get(self.downloadUrl, headers=headers, stream=True)
      response.raise_for_status()  # vyvolá výjimku při špatném HTTP statusu
    except requests.RequestException as e:
      log.error(f"Error downloading audio file: {e}")
      return
    chunkSize = 8192
    toneCounter = 0
    try:
      with open(self.downloadTarget, "wb") as outFile:
        for chunk in response.iter_content(chunk_size=chunkSize):
          if not chunk:
            break
          outFile.write(chunk)
          toneCounter += 1
          if toneCounter >= 1000:
            toneCounter = 0
            if not self.quiet:
              tones.beep(440, 40)
        if not self.quiet:
          tones.beep(880, 40)
    except Exception as e:
      log.error(f"Error writing to audio file: {e}")

  def startDownload(self):
    self._shouldCancel = False
    self._guiExecTimer = gui.NonReEntrantTimer(self._guiExecNotify)
    # Translators: title of progress dialog - download
    progTitle = _("Downloading")
    # Translators: message of progress dialog - connecting
    progMessage = _("Connecting")

    self._progressDialog = wx.ProgressDialog(progTitle,
      progMessage,
      style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE,
      )
    self._progressDialog.CentreOnScreen()
    self._progressDialog.Raise()
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.startDownload.__qualname__}",
      target=self._bg
    )
    t.daemon = True
    t.start()

  def _guiExec(self, func, *args):
    self._guiExecFunc = func
    self._guiExecArgs = args
    if not self._guiExecTimer.IsRunning():
      wx.CallAfter(self._guiExecTimer.Start, 50, True)

  def _guiExecNotify(self):
    self._guiExecFunc(*self._guiExecArgs)

  def _bg(self):
    success=False
    try:
      self._download()
    except:
      log.error("Chyba při stahování")
    else:
      if not self._shouldCancel:
        success=True
    if not success:
      self._guiExec(self._error)
      return
    self._guiExec(self._downloadSuccess)

  def _download(self):
    headers = {
      'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
    }
    try:
      request = urllib.request.Request(self.downloadUrl)
      remote = urllib.request.urlopen(request)
    except:
      request = urllib.request.Request(self.downloadUrl, None, headers)
      remote = urllib.request.urlopen(request)
    if remote.code != 200:
      raise RuntimeError("Stahování selhalo s chybou %d" % remote.code)
    size = int(remote.headers["content-length"])
    with open(self.downloadTarget, "wb") as local:
      self._guiExec(self._downloadReport, 0, size)
      read = 0
      chunk=8192
      while True:
        if self._shouldCancel:
          return
        if size -read <chunk:
          chunk =size -read
        block = remote.read(chunk)
        if not block:
          break
        read += len(block)
        if self._shouldCancel:
          return
        local.write(block)
        self._guiExec(self._downloadReport, read, size)
      if read < size:
        raise RuntimeError("Stahovaný obsah je příliš krátký")
    self._guiExec(self._downloadReport, read, size)

  def _downloadReport(self, read, size):
    if self._shouldCancel:
      return
    percent = int(float(read) / size * 100)
    # Translators: text saying that the file is being downloaded
    cont, skip = self._progressDialog.Update(percent, _("Downloading"))
    if not cont:
      self._shouldCancel = True
      self._stopped()

  def _stopped(self):
    self._guiExecTimer = None
    self._guiExecFunc = None
    self._guiExecArgs = None
    self._progressDialog.Hide()
    self._progressDialog.Destroy()
    self._progressDialog = None

  def _error(self):
    self._stopped()
    # Translators: message that file cannot be downloaded
    errorMessage = _("File can not be downloaded.")
    # Translators: title of the dialog - warning
    errorTitle = _("Warning")
    wx.MessageBox(errorMessage, errorTitle)

  def _downloadSuccess(self):
    self._stopped()
    # start addon update
    if self.callingProcess == "addon update":
      os.startfile(self.downloadTarget)

    # end of download with progressbar

class VirtualListCtrl(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    
  def __init__(self, parent, columnTitle, data, selectionStyle=0):
    wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT | wx.LC_VIRTUAL | wx.LC_NO_HEADER | selectionStyle)
    listmix.ListCtrlAutoWidthMixin.__init__(self)
    self.InsertColumn(0, columnTitle)        
    self.data = data
    self.SetItemCount(len(self.data))    
    self.setResizeColumn(0)

  def OnGetItemText(self, item, column):
    return self.data[item]
    
  def insertList(self, newList):
    self.data = newList[:]
    self.SetItemCount(len(self.data))

  def insertItems(self, index, items):
    self.data[index:index] = items
    self.SetItemCount(len(self.data))

  def getSelectedItem(self):
    return self.data[self.GetFirstSelected()]

  def getSelectedItems(self):
    selected_items = []
    current = self.GetFirstSelected()
    while current != -1:
      selected_items.append(self.data[current])
      current = self.GetNextSelected(current)
    return selected_items

  def getLastSelectedItem(self):
    selected_indices = self.getSelectedIndices()
    if selected_indices:
      return self.data[selected_indices[-1]]
    else:
      return None
  
  def getSelectedSmartItem(self):
    """returns first item if only one is selected otherwise returns last selected item """
    selected_indices = self.getSelectedIndices()
    if selected_indices:
      if len(selected_indices) == 1:
        # returns first item which is selected as only one
        return self.data[selected_indices[0]]
      else:
        # returns last selected item
        return self.data[selected_indices[-1]]
    else:
      return None

  def getSelectedIndices(self):
    selected_indices = []
    current = self.GetFirstSelected()
    while current != -1:
      selected_indices.append(current)
      current = self.GetNextSelected(current)
    return selected_indices

  def getAllItems(self):
    return self.data[:]

  def getItem(self, index):
    if 0 <= index < len(self.data):
      return self.data[index]
    else:
      return None

  def selectItem(self, itemIndex):
    self.Select(itemIndex)
    self.Focus(itemIndex)

  def deleteItem(self, index):
    del self.data[index]
    self.SetItemCount(len(self.data))    

  def addItem(self, index, itemText):
    self.data.insert(index, itemText)
    self.SetItemCount(len(self.data))

  def clearAllItems(self):
    self.data.clear()
    self.SetItemCount(0)

  def findString(self, string, caseSensitive=False):
    if not caseSensitive:
      string = string.lower()
    for index, item in enumerate(self.data):
      text = item
      if not caseSensitive:
        text = text.lower()
      if string in text:
        return index
    return wx.NOT_FOUND

class VirtualListCtrlUid(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    
  def __init__(self, parent, columnTitle, data, selectionStyle=0):
    wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT | wx.LC_VIRTUAL | wx.LC_NO_HEADER | selectionStyle)
    listmix.ListCtrlAutoWidthMixin.__init__(self)
    self.InsertColumn(0, columnTitle)        
    self._itemDataMap = {}
    self.data = []
    for index, item in enumerate(data):
      self.data.append(item['title'])
      self._itemDataMap[index] = item['id']
    self.SetItemCount(len(self.data))    
    self.setResizeColumn(0)

  def OnGetItemText(self, item, column):
    return self.data[item]
    
  def GetItemIdFromIndex(self, index):
    return self._itemDataMap.get(index)

  def GetIndexFromItemId(self, item_id):
    for index, _id in self._itemDataMap.items():
      if _id == item_id:
        return index
    return -1

  def insertList(self, newList):
    self.data = []
    self._itemDataMap = {}
    for index, item in enumerate(newList):
      self.data.append(item['title'])
      self._itemDataMap[index] = item['id']
    self.SetItemCount(len(self.data))

  def insertItems(self, index, items):
    self.data[index:index] = items
    self.SetItemCount(len(self.data))

  def getSelectedItem(self):
    return self.data[self.GetFirstSelected()]

  def getSelectedItems(self):
    selected_items = []
    current = self.GetFirstSelected()
    while current != -1:
      selected_items.append(self.data[current])
      current = self.GetNextSelected(current)
    return selected_items

  def getLastSelectedItem(self):
    selected_indices = self.getSelectedIndices()
    if selected_indices:
      return self.data[selected_indices[-1]]
    else:
      return None
  
  def getSelectedSmartItem(self):
    """returns first item if only one is selected otherwise returns last selected item """
    selected_indices = self.getSelectedIndices()
    if selected_indices:
      if len(selected_indices) == 1:
        # returns first item which is selected as only one
        return self.GetItemIdFromIndex(selected_indices[0])
      else:
        # returns last selected item
        return self.GetItemIdFromIndex(selected_indices[-1])
    else:
      return None

  def getSelectedIndices(self):
    selected_indices = []
    current = self.GetFirstSelected()
    while current != -1:
      selected_indices.append(current)
      current = self.GetNextSelected(current)
    return selected_indices

  def getAllItems(self):
    return self.data[:]

  def getItem(self, index):
    if 0 <= index < len(self.data):
      return self.data[index]
    else:
      return None

  def selectItem(self, itemIndex):
    self.Select(itemIndex)
    self.Focus(itemIndex)

  def deleteItem(self, index):
    item_id = self._itemDataMap.pop(index)  # Odstraníme položku z _itemDataMap a získáme její ID
    del self.data[index]
    self.SetItemCount(len(self.data))    
    self._itemDataMap = {new_index: _id for new_index, _id in enumerate(self._itemDataMap.values())}
  
  def addItem(self, index, itemText):
    self.data.insert(index, itemText)
    self.SetItemCount(len(self.data))

  def clearAllItems(self):
    self.data.clear()
    self.SetItemCount(0)

  def findString(self, string, caseSensitive=False):
    if not caseSensitive:
      string = string.lower()
    for index, item in enumerate(self.data):
      text = item
      if not caseSensitive:
        text = text.lower()
      if string in text:
        return index
    return wx.NOT_FOUND


class RssTaskBarIcon(wx.adv.TaskBarIcon):
  
  def __init__(self, frame):
    super(RssTaskBarIcon, self).__init__()
    self.frame = frame

    # set icon
    self.SetIcon(icon, "")

    # tray menu
    self.Bind(wx.adv.EVT_TASKBAR_LEFT_DOWN, self.trayLeftClick)
    self.Bind(wx.adv.EVT_TASKBAR_RIGHT_DOWN, self.trayRightClick)

  def trayLeftClick(self, evt):
    # restore window
    if self.frame.IsIconized():
      self.frame.Iconize(False)
    if not self.frame.IsShown():
      self.frame.Show()
    self.frame.Raise()

  def trayRightClick(self, evt):
    # create context menu
    menu = wx.Menu()
    # Translators: systemtray menu item to display RSS reader GUI
    restore_item = wx.MenuItem(menu, wx.ID_ANY, _("&Show RSS reader"))
    # Translators: systemtray menu item to exit RSS reader
    exit_item = wx.MenuItem(menu, wx.ID_EXIT, _("&Exit RSS reader"))
    menu.Append(restore_item)
    menu.AppendSeparator()
    menu.Append(exit_item)
    self.Bind(wx.EVT_MENU, self.on_restore, restore_item)
    self.Bind(wx.EVT_MENU, self.on_exit, exit_item)
    self.PopupMenu(menu)

  def on_restore(self, event):
    # restore window
    self.frame.Show()
    self.frame.Restore()

  def on_exit(self, event):
    # exit app
    wx.CallAfter(self.Destroy)
    self.frame.Close


class TextWindow(wx.Frame):

  dispText = ""
  dlgTitle = ""
  sayAllRunning = False
  findData = None
  passKey = False

  def __init__(self, parent):
    # the title of text window
    super(TextWindow, self).__init__(parent, size=(1000, 800), title=self.dlgTitle)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    self.Bind(wx.EVT_FIND, self.onFind)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
      
    # menu items
    # Translators: menu item to start or stop reading
    item = menu.Append(wx.ID_ANY, _("Start or stop reading	Space"))
    self.Bind(wx.EVT_MENU, self.startstopReading, item)
    # Translators: Menu item to search text
    item = menu.Append(wx.ID_ANY, _("Search	Ctrl+F"))
    self.Bind(wx.EVT_MENU, self.searchText, item)
    # Translators: Menu item to find next
    item = menu.Append(wx.ID_ANY, _("Find next	F3"))
    self.Bind(wx.EVT_MENU, self.findNext, item)
    # Translators: Menu item to find previous
    item = menu.Append(wx.ID_ANY, _("Find previous	Shift+F3"))
    self.Bind(wx.EVT_MENU, self.findPrevious, item)
    # Translators: Menu item to display message in web browser
    item = menu.Append(wx.ID_ANY, _("Display message in web browser	F5"))
    self.Bind(wx.EVT_MENU, self.displayinBrowserItem, item)
    menu.AppendSeparator()
    # Translators: Menu item to close text viewer
    item = menu.Append(wx.ID_EXIT, _("&Close"))
    self.Bind(wx.EVT_MENU, self.onClose, item)

    # the menu title
    # Translators: Text viewer main menu item
    menuBar.Append(menu, _("Text viewer"))
    self.SetMenuBar(menuBar)

    # the edit field
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)
    self.outputCtrl.Bind(wx.EVT_KEY_UP, self.onOutputKeyUp)
    sHelper.addItem(self.outputCtrl, proportion=1, flag=wx.EXPAND)

    self.outputCtrl.SetValue(self.dispText)
    self.outputCtrl.SetFocus()

  def startReading(self):
    speech.cancelSpeech()
    if not self.sayAllRunning:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def startstopReading(self, evt):
    if self.sayAllRunning:
      self.sayAllRunning = False
      sayAll.SayAllHandler.stop()
    else:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def jumptoPlace(self, place):
    # move cursor to the found text position
    self.outputCtrl.SetInsertionPoint(place)
    # and scroll to this position
    self.outputCtrl.ShowPosition(place)
    self.outputCtrl.Refresh()
    self.outputCtrl.Update()

  def searchText(self, evt):
    self.findData = wx.FindReplaceData()
    # Translators: title of search text dialog
    self.searchDlg = wx.FindReplaceDialog(self, self.findData, title=_("Search text"), style=wx.FR_NOUPDOWN | wx.FR_NOMATCHCASE | wx.FR_NOWHOLEWORD)
    self.searchDlg.Show()
    self.passKey = True

  def onFind(self, event):
    if not self.findData:
      self.searchText(None)
    else:
      fstring = self.findData.GetFindString().upper()
      self.searchDlg.Close()
      sText = self.outputCtrl.GetValue().upper()
      currentFindPos = self.outputCtrl.GetInsertionPoint()
      (par, charPos, lineNumber) = self.outputCtrl.PositionToXY(currentFindPos)
      i = lineNumber
      found = False
      while i <= self.outputCtrl.GetNumberOfLines():
        line = self.outputCtrl.GetLineText(i).upper()
        if i == lineNumber:
          startPos = line.find(fstring, charPos+1)
        else:
          startPos = line.find(fstring)
        if startPos != -1:
          found = True
          jumpPos = self.outputCtrl.XYToPosition(startPos, i)
          self.jumptoPlace(jumpPos)
          speech.speakMessage(line)
          break
        i = i +1
      if not found:
        # Translators: error message, text was not found
        errorMessage = _("Text %s was not found.") % fstring
        # Translators: title of warning dialog
        errorTitle = _("Warning")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True

  def findNext(self, evt):
    self.onFind(self)

  def findPrevious(self, evt):
    if not self.findData:
      self.searchText(None)
    else:
      fstring = self.findData.GetFindString().upper()
      sText = self.outputCtrl.GetValue().upper()
      currentFindPos = self.outputCtrl.GetInsertionPoint()
      (par, charPos, lineNumber) = self.outputCtrl.PositionToXY(currentFindPos)
      i = lineNumber
      found = False
      while i >= 0:
        line = self.outputCtrl.GetLineText(i).upper()
        if i == lineNumber:
          startPos = line.rfind(fstring, 0, charPos)
        else:
          startPos = line.rfind(fstring)
        if startPos != -1:
          found = True
          jumpPos = self.outputCtrl.XYToPosition(startPos, i)
          # move cursor to the found text position
          self.outputCtrl.SetInsertionPoint(jumpPos)
          # and scroll to this position
          self.outputCtrl.ShowPosition(jumpPos)
          self.outputCtrl.Refresh()
          self.outputCtrl.Update()
          speech.speakMessage(line)
          break
        i = i -1
      if not found:
        # Translators: searched text was not found
        errorMessage = _("Text %s was not found.") % fstring 
        # Translators: title of warning dialog
        errorTitle = _("Warning")
        wx.MessageBox(errorMessage, errorTitle)
        self.passKey = True

  def displayinBrowserItem(self, evt):
    index = self.Parent.getSelectedArticleIndex()
    articleAddress = self.Parent.Parent.channel.getArticleLink(index)
    os.startfile(articleAddress)
    self.Close()

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()

  def onOutputKeyUp(self, evt):
    key = evt.GetKeyCode()
    if self.passKey:
      self.passKey = False
      return
    if (key == wx.WXK_ESCAPE) or (key == wx.WXK_BACK):
      self.Close()
    elif key != wx.WXK_SPACE:
      self.sayAllRunning = False
    evt.Skip()


class AudioPlayer(wx.Frame):

  dlgTitle = ""
  playingPos = 0
  source = ""
  currentVolume = 100
  currentRate = 1.0
  rateSpeech = {
    # Translators: playing rate 0.5
    0.5: _("Half"),
    # Translators: playing rate 0.75
    0.75: _("Three-quarter"),
    # Translators: playing rate 1.0
    1.0: _("Normal"),
    # Translators: playing rate 1.25
    1.25: _("One and a quarter"),
    # Translators: playing rate 1.5
    1.5: _("One and a half"),
    # Translators: playing rate 1.75
    1.75: _("One and a three-quarter"),
    # Translators: playing rate 2.0
    2.0: _("Double")
  }
  playingEnded = False
  if vlcInstalled:
    vlc_instance = vlc.Instance()
    player = vlc_instance.media_player_new()
    events = player.event_manager()

  def __init__(self, parent):
    # The title of the AudioPlayer dialog
    # Translators: Title of audioplayer window
    super(AudioPlayer, self).__init__(parent, size=(1000, 800), title=_("Audioplayer - %s") % self.dlgTitle)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    panel = wx.Panel(self)
    # standard self.Bind(wx.EVT_KEY_UP, self.audioPlayerKeyUp) does not work
    accel_tbl = wx.AcceleratorTable([
      (wx.ACCEL_NORMAL, wx.WXK_BACK, wx.ID_ANY),
      (wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_ANY)
    ])
    self.SetAcceleratorTable(accel_tbl)
    self.Bind(wx.EVT_MENU, self.audioPlayerKeyUp, id=wx.ID_ANY)
    
    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    # Translators: Menu item to play or stop audio playback
    item = menu.Append(wx.ID_ANY, _("Start or stop &playing	Space"))
    self.Bind(wx.EVT_MENU, self.startstopPlaying, item)
    # Translators: Menu item to save audio file
    item = menu.Append(wx.ID_ANY, _("&Save audio file	Ctrl+S"))
    self.Bind(wx.EVT_MENU, self.saveAudioFile, item)
    # Translators: Menu item to open folder with saved audio files
    item = menu.Append(wx.ID_ANY, _("&Open folder with stored audio files"))
    self.Bind(wx.EVT_MENU, self.openAudioFolder, item)
    # Translators: Menu item to go to begining of audio file
    item = menu.Append(wx.ID_ANY, _("Move to &Begining	Home"))
    self.Bind(wx.EVT_MENU, self.gotoStart, item)
    # Translators: Menu item to go to end of audio file
    item = menu.Append(wx.ID_ANY, _("Move to &End	End"))
    self.Bind(wx.EVT_MENU, self.gotoEnd, item)
    # Translators: Menu item to announce current playback position
    item = menu.Append(wx.ID_ANY, _("Announce current position	F2"))
    self.Bind(wx.EVT_MENU, self.announceCurrentPosition, item)
    # Translators: Menu item to move 10 seconds forward
    item = menu.Append(wx.ID_ANY, _("10 seconds forward	Right"))
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(10000), item)
    # Translators: Menu item to move 10 seconds backward
    item = menu.Append(wx.ID_ANY, _("10 seconds backward	Left"))
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(10000), item)
    # Translators: Menu item to move one minute forward
    item = menu.Append(wx.ID_ANY, _("1 minute forward	Ctrl+Right"))
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(60000), item)
    # Translators: Menu item to move one minute backward
    item = menu.Append(wx.ID_ANY, _("1 minute backward	Ctrl+Left"))
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(60000), item)
    # Translators: Menu item to move 3 seconds forward
    item = menu.Append(wx.ID_ANY, _("3 second forward	Shift+Right"))
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(3000), item)
    # Translators: Menu item to move 3 seconds backward
    item = menu.Append(wx.ID_ANY, _("3 seconds left	Shift+Left"))
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(3000), item)
    # Translators: Menu item to move 10 minutes forward
    item = menu.Append(wx.ID_ANY, _("10 minutes forward	Alt+Right"))
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(600000), item)
    # Translators: Menu item to move 10 minutes backward
    item = menu.Append(wx.ID_ANY, _("10 minutes backward	Alt+Left"))
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(600000), item)
    # Translators: Menu item to move 5 minutes forward
    item = menu.Append(wx.ID_ANY, _("5 minutes forward	Ctrl+Alt+Right"))
    self.Bind(wx.EVT_MENU, lambda event: self.goForward(300000), item)
    # Translators: Menu item to move 5 minutes backward
    item = menu.Append(wx.ID_ANY, _("5 minutes left	Ctrl+Alt+Left"))
    self.Bind(wx.EVT_MENU, lambda event: self.goBackward(300000), item)
    # Translators: Menu item to increase volume
    item = menu.Append(wx.ID_ANY, _("Volume up	Up"))
    self.Bind(wx.EVT_MENU, lambda event: self.changeVolume(5), item)
    # Translators: Menu item to decrease volume
    item = menu.Append(wx.ID_ANY, _("Volume down	Down"))
    self.Bind(wx.EVT_MENU, lambda event: self.changeVolume(-5), item)
    # Translators: Menu item to increase playing rate
    item = menu.Append(wx.ID_ANY, _("Faster	Ctrl+Up"))
    self.Bind(wx.EVT_MENU, lambda event: self.changeRate(0.25), item)
    # Translators: Menu item to decrease playing rate
    item = menu.Append(wx.ID_ANY, _("Slower	Ctrl+Down"))
    self.Bind(wx.EVT_MENU, lambda event: self.changeRate(-0.25), item)
    # Translators: Menu item to open playing media in player (VLC player)
    item = menu.Append(wx.ID_ANY, _("Open in &VLC	F5"))
    self.Bind(wx.EVT_MENU, self.openinVlcItem, item)
    menu.AppendSeparator()
    # Translators: menu title to exit audioplayer
    item = menu.Append(wx.ID_EXIT, _("&Close"))
    self.Bind(wx.EVT_MENU, self.onCloseItem, item)
    
    # the menu title
    # Translators: title of main audioplayer menu
    menuBar.Append(menu, _("Audioplayer"))
    self.SetMenuBar(menuBar)

    panel.SetFocus()
    menuBar.EnableTop(0, True)  
    self.media = self.vlc_instance.media_new(self.source)
    self.player.set_media(self.media)
    self.events.event_attach(vlc.EventType.MediaPlayerEndReached, self.playingFinished)
    self.playingEnded =False
    self.Parent.Parent.quietMode = True
    self.player.play()
    # move to the last playing position
    self.player.set_position(self.playingPos)

  def openAudioFolder(self, evt):
    os.startfile(podcastsMp3Path)

  def saveAudioFile(self, evt):
    index = self.Parent.getSelectedArticleIndex()
    saveResult = self.Parent.Parent.channel.saveAudio(index, force=False, progress=True)
    if saveResult != "":
      if gui.messageBox(
        # Translators: question that this sound file is already downloaded, do you want to download it again?
        _("This audio file has already been downloaded. Would you like to download it again?"),
        # Translators: title of warning dialog with question
        _("Warning"),
        wx.YES | wx.NO | wx.ICON_QUESTION, self
      ) == wx.YES:
        saveResult = self.Parent.Parent.channel.saveAudio(index, force=True, progress=True)

  def openinVlcItem(self, evt):
    openinVlc(self.source)
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()

  def playingFinished(self, event):
    if not self.playingEnded:
      self.playingEnded = True
      self.Parent.Parent.quietMode = False
      # Translators: speech message that playing was finished
      speech.speakMessage(_("Playing was finished."))

  def playingTime(self, pTime):
    curTime = datetime.datetime.utcfromtimestamp(pTime/1000.0)
    strTime = ""
    if curTime.hour > 0:
      strTime = strTime + str(curTime.hour)
      # Translators: string for number of hours
      strTime = strTime + ngettext(" hour", " hours", curTime.hour)
      strTime = strTime + ", "
    if curTime.minute > 0:
      strTime = strTime + str(curTime.minute)
      # Translators: string for number of minutes
      strTime = strTime + ngettext(" minute", " minutes", curTime.minute)
    if strTime:
      # Translators: conjunction and before seconds
      strTime = strTime + _(" and ")
    strTime = strTime + str(curTime.second)
    # Translators: string for number of seconds
    strTime = strTime + ngettext(" second", " seconds", curTime.second)
    return strTime

  def startstopPlaying(self, evt):
    state = self.player.get_state()
    if state == vlc.State.Ended:
      self.player.stop()
      self.player.set_position(0.0)
      self.playingEnded = False
      self.Parent.Parent.quietMode = True
      self.player.play()
    elif state == vlc.State.Stopped:
      self.playingEnded = False
      self.Parent.Parent.quietMode = True
      self.player.play()
    else:
      self.player.pause()
      if self.Parent.Parent.quietMode == True:
        self.Parent.Parent.quietMode = False
      else:
        self.Parent.Parent.quietMode = True


  def gotoStart(self, evt):
    self.player.set_position(0)
    speech.cancelSpeech()
    # Translators: begining of audio file message
    ui.message(_("Begining."))

  def gotoEnd(self, evt):
    endTime = self.player.get_length()
    if self.player.is_playing():
      self.player.pause()
    self.player.set_time(endTime)
    speech.cancelSpeech()
    # Translators: end of audio file message
    ui.message(_("End: " + self.playingTime(endTime) + "."))

  def announceCurrentPosition(self, evt):
    endTime = self.player.get_length()
    curTime = self.player.get_time()
    remTime = endTime - curTime
    speech.cancelSpeech()
    # Translators: passed playing time and remaining playing time
    ui.message(_("Elapsed time: %s, remaining time: %s.") % (self.playingTime(curTime), self.playingTime(remTime)))

  def goForward(self, timeOffset):
    endTime = self.player.get_length()
    curTime = self.player.get_time()
    curTime = curTime + timeOffset
    if curTime > endTime:
      curTime = endTime
    self.player.set_time(curTime)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(self.playingTime(curTime) + ".")

  def goBackward(self, timeOffset):
    curTime = self.player.get_time()
    curTime = curTime - timeOffset
    if curTime < 0:
      curTime = 0
    self.player.set_time(curTime)
    if not self.player.is_playing():
      speech.cancelSpeech()
      ui.message(self.playingTime(curTime) + ".")

  def changeRate(self, rate):
    self.currentRate = self.currentRate + rate
    if self.currentRate > 2.0:
      self.currentRate = 2.0
    if self.currentRate < 0.5:
      self.currentRate = 0.5
    self.player.set_rate(self.currentRate)
    if not self.player.is_playing():
      speech.cancelSpeech()
      # Translators: message announced after set playing rate
      ui.message(_("%s playing rate.") % self.rateSpeech[self.currentRate])

  def changeVolume(self, volume):
    self.currentVolume = self.currentVolume + volume
    if self.currentVolume > 100:
      self.currentVolume = 100
    if self.currentVolume < 0:
      self.currentVolume = 0
    self.player.audio_set_volume(self.currentVolume)
    if not self.player.is_playing():
      speech.cancelSpeech()
      # Translators: message announced after set playing volume
      ui.message(_("%s is volume.") % str(self.currentVolume))

  def audioPlayerKeyUp(self, evt):
    # performs according to acceleratortable on Backspace and Escape
    self.onClose(self)

  def saveListeningPosition(self):
    index = self.Parent.getSelectedArticleIndex()
    posMark = self.Parent.Parent.channel.articles[index].find(self.Parent.Parent.channel.applyNs("audioposition", self.Parent.Parent.channel.ns))
    posMark.text = str(self.playingPos)
    self.Parent.needSave = True
  
  def onCloseItem(self, evt):
    self.onClose(self)

  def onClose(self, evt):
    self.playingPos = self.player.get_position()
    self.saveListeningPosition()
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()
    self.player.stop()


class RssReader(wx.Frame):

  appdataFolder = os.getenv('APPDATA')
  opmlTree = None
  channel = None
  readingChannel = None
  newChannelName = None
  dbSelectedUrl = ""
  inArchive = False
  # Translators: title of archive item in the list of channels
  archiveTitle = _("Archive")
  newsChannels = dict()
  filteredNews = False
  searchResults = dict()
  # Translators: Title of item for displaying all channels in the channels list
  displayAllChannelsTitle = _("Display all feeds")
  # Translators: Item for displaying all messages from all channels in the channels list
  displayAllMessagesTitle = _("All")
  quietMode = False
  if not os.path.exists(os.path.join(appdataFolder, "Marsus")):
    os.mkdir(os.path.join(appdataFolder, "Marsus"))
  if not os.path.exists(os.path.join(appdataFolder, "Marsus", "NVDA")):
    os.mkdir(os.path.join(appdataFolder, "Marsus", "NVDA"))
  if not os.path.exists(RssFolder):
    os.mkdir(RssFolder)
  if not os.path.exists(articlesFolder):
    os.mkdir(articlesFolder)
  # create OPML file
  if not os.path.isfile(opmlFilePath):
    try:
      opml = ET.Element("opml")
      opml.set("version", "2.0")
      head = ET.Element("head")
      title = ET.Element("title")
      title.text = mainTitle
      head.append(title)
      opml.append(head)
      body = ET.Element("body")
      opml.append(body)
      with open(opmlFilePath, 'wb') as f:
        f.write(ET.tostring(opml, encoding='utf-8', method='xml'))
    except Exception as e:
      raise e

  _instance = None

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if RssReader._instance is None:
      return super(RssReader, cls).__new__(cls, *args, **kwargs)
    return RssReader._instance

  def __init__(self, parent, minimized=False):
    if RssReader._instance is not None:
      return
    RssReader._instance = self

    # The window title
    super(RssReader, self).__init__(parent, size=(1000, 800), title=mainTitle)
    global channelHwnd
    channelHwnd = self.GetHandle()
    self.articlesWindow = None

    # icon
    self.SetIcon(icon)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    self.taskbarIcon = None
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.choices = []

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    # Translators: Menu item to add new channel
    item = menu.Append(wx.ID_ANY, _("&New feed	Ctrl+N"))
    self.Bind(wx.EVT_MENU, self.newChannelItem, item)
    # Translators: Menu item to add channel from database
    item = menu.Append(wx.ID_ANY, _("Add channel from &database	Ctrl+D"))
    self.Bind(wx.EVT_MENU, self.channelFromDatabaseItem, item)
    # Translators: Menu item to display channel properties
    item = menu.Append(wx.ID_ANY, _("Channel &Properties	Alt+Enter"))
    self.Bind(wx.EVT_MENU, self.channelPropertiesItem, item)
    # Translators: Menu item to delete channel
    item = menu.Append(wx.ID_ANY, _("&Delete feed	Delete"))
    self.Bind(wx.EVT_MENU, self.deleteChannelItem, item)
    # Translators: Menu item to import channels from OPML file
    item = menu.Append(wx.ID_ANY, _("&Import feeds from OPML file"))
    self.Bind(wx.EVT_MENU, self.importChannelsFromOpmlItem, item)
    # Translators: Menu item to export channels to OPML file
    item = menu.Append(wx.ID_ANY, _("&Export channels to OPML file"))
    self.Bind(wx.EVT_MENU, self.exportChannelsToOpmlItem, item)
    # Translators: Menu item to read all new messages from all channels
    item = menu.Append(wx.ID_ANY, _("Read &new messages from all feeds	F5"))
    self.Bind(wx.EVT_MENU, self.readAllNewArticlesItem, item)
    # Translators: Menu item to read all new messages from selected channel
    item = menu.Append(wx.ID_ANY, _("Read new messages from &selected feed	Ctrl+F5"))
    self.Bind(wx.EVT_MENU, self.readSelectedNewArticlesItem, item)
    # Translators: Menu item to switch filter displaying only channels with new messages
    self.newsMenuItem = menu.AppendCheckItem(wx.ID_ANY, _("Display &only feeds with new messages	F4"))
    self.Bind(wx.EVT_MENU, self.displayChannelsFilterItem, self.newsMenuItem)
    # Translators: Menu item to open a list of messages
    item = menu.Append(wx.ID_ANY, _("Open message &list	Enter"))
    self.Bind(wx.EVT_MENU, self.openArticlesListItem, item)
    # Translators: Menu item to announce number of messages in selected channel
    item = menu.Append(wx.ID_ANY, _("Announce the number of &messages in selected feed	F2"))
    self.Bind(wx.EVT_MENU, self.sayArticlesNumberItem, item)
    # Translators: Menu item to search text in messages of all channels
    item = menu.Append(wx.ID_ANY, _("Search &text in messages of all feeds	Ctrl+F"))
    self.Bind(wx.EVT_MENU, self.searchItem, item)
    # Translators: Menu item to open archive
    item = menu.Append(wx.ID_ANY, _("Open &archive	Ctrl+F6"))
    self.Bind(wx.EVT_MENU, self.openArchiveItem, item)
    # Translators: Menu item to go back from archive
    item = menu.Append(wx.ID_ANY, _("&Back from archive	Backspace"))
    self.Bind(wx.EVT_MENU, self.backFromArchiveItem, item)
    # Translators: menu item to search podcasts in Apple podcasts
    item = menu.Append(wx.ID_ANY, _("Search &podcasts in Apple podcasts	Ctrl+H"))
    self.Bind(wx.EVT_MENU, self.searchItunesPodcastsItem, item)
    # Translators: Menu item to open settings for selected channel
    item = menu.Append(wx.ID_ANY, _("&Settings for selected feed	Ctrl+F12"))
    self.Bind(wx.EVT_MENU, self.settingsForChannelItem, item)
    # Translators: Menu item to open application settings
    item = menu.Append(wx.ID_ANY, _("S&ettings	F12"))
    self.Bind(wx.EVT_MENU, self.settingsItem, item)
    # Translators: Menu item to open help
    item = menu.Append(wx.ID_ANY, _("&Help"))
    self.Bind(wx.EVT_MENU, self.helpItem, item)
    # Translators: Menu item to display dialog about application
    item = menu.Append(wx.ID_ANY, _("&About"))
    self.Bind(wx.EVT_MENU, self.aboutItem, item)
    menu.AppendSeparator()
    # Translators: Menu item to exit application
    item = menu.Append(wx.ID_EXIT, _("&Exit"))
    self.Bind(wx.EVT_MENU, self.exitItem, item)
    
    # the menu title
    menuBar.Append(menu, mainTitle)
    self.SetMenuBar(menuBar)

    # load OPML file
    self.loadOpmlChannels()

    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    # Translators: Title in the channels list column
    self.channelsList = VirtualListCtrl(self, _("Feeds"), self.choices, selectionStyle=wx.LC_SINGLE_SEL)
    self.channelsList.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.onchannelsListChange)
    self.channelsList.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onchannelsListChoice)
    self.channelsList.Bind(wx.EVT_LIST_KEY_DOWN, self.onchannelsListKeyDown)
    self.channelsList.SetMinSize((1000, 800))
    sHelper.addItem(self.channelsList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.channelsList.SetFocus()
    self.CentreOnScreen()
    if self.channelsList.GetItemCount() > 0:
      self.channelsList.selectItem(0)
      if downloadArticlesAfterStartup:
        # the timer which will start downloading new messages shortly after startup
        self.timer = wx.Timer(self)
        self.timer.StartOnce(2000)
        self.Bind(wx.EVT_TIMER, self.readAllNewArticlesItem)
    else:
      wx.CallAfter(self.noChannels)
    self.addArchiveToList()
    # the timer which will download new messages in regular interval
    self.regularTimer = wx.Timer(self)
    self.Bind(wx.EVT_TIMER, self.readAllNewArticlesItem)
    if downloadArticlesInterval > 0:
      self.regularTimer.Start(convertDownloadArticlesInterval(downloadArticlesInterval))
    # minimize to tray if turn on in settings and interval is set
    if minimized:
      if not self.taskbarIcon:
        self.taskbarIcon = RssTaskBarIcon(self)

  def onGetItem(self, evt):
    index = evt.GetIndex()
    item = wx.ListItem()
    item.SetId(index)
    item.SetText(self.choices[index], 0)

  def searchItunesPodcastsItem(self, evt):
    # Translators: Search in Apple podcasts dialog title
    dlgTitle = _("Searching in Apple Podcasts")
    # Translators: Search in Apple podcasts dialog message
    dlgMessage = _("Enter search text in Apple Podcasts:")
    with wx.TextEntryDialog(self, dlgMessage, dlgTitle) as d:
      if d.ShowModal() == wx.ID_CANCEL or not d.Value:
        self.channelsList.SetFocus()
        return
    foundPodcasts = podsearch.search(d.Value, country="cz", limit=50)
    if len(foundPodcasts) == 0:
      # Translators: no search results in Apple podcasts dialog message
      dlgMessage = _("No results found for the search text %s." % d.Value)
      # Translators: no search results in Apple podcasts dialog title
      dlgTitle = _("Information")
      wx.MessageBox(dlgMessage, dlgTitle)
      return
    self.dbSelectedUrl = ""
    self.Hide()
    self.Disable()
    try:
      PodcastSearchDialog(self, foundPodcasts).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if not self.dbSelectedUrl:
      return
    name = self.createChannel(self.dbSelectedUrl)
    if name:
      self.choices.append(name)
      self.choices.sort(key=locale.strxfrm)
      self.channelsList.insertList(self.choices)
      self.addArchiveToList()
      chIndex = self.channelsList.findString(name)
      self.channelsList.selectItem(chIndex)
      self.channelsList.SetFocus()

  def addUid(self, chname):
    channel = RssChannel()
    channel.setChannelName(chname)
    fType = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == chname:
        fType = outline.get("type")
        break
    channel.setFeedType(fType)
    channel.loadArticlesFromFile(self.inArchive)
    channel.writeArticles(self.inArchive)

  def searchItem(self, evt):
    if self.channelsList.GetItemCount() == 0:
      return
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      dlgMessage = inprocessMessage
      # Translators: title of warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    global searchFilter
    # Translators: Title of dialog for searching text in all messages of all feeds
    dlgTitle = _("Search text in messages of all feeds")
    # Translators: Message of dialog for searching text in all messages of all channels
    dlgMessage = _("Enter text to search:")
    with wx.TextEntryDialog(self, dlgMessage, dlgTitle, value="") as d:
      if d.ShowModal() == wx.ID_CANCEL or not d.Value:
        return
      # searching in all channels and articles
      self.searchResults.clear()
      for file in os.listdir(articlesFolder):
        curFile = os.path.join(articlesFolder, file)
        if os.path.isfile(curFile):
          artResultDict = {}
          missingUid = False
          with open(curFile, 'r', encoding="utf8", newline='') as f:
            reader = csv.DictReader(f)
            for row in reader:
              # when searched string will be in title or in description
              if (d.Value.upper() in row["title"].upper()) or (d.Value.upper() in row["description"].upper()):
                # and when the message is not deleted
                if row["deleted"] == "0":
                  try:
                    artResultDict[row["title"]] = row["uid"]
                  except:
                    missingUid = True
                    artResultDict.clear()
                    break
          if missingUid:
            # uid is missing, add it to all articles in channel and search it again
            self.addUid(os.path.splitext(file)[0])
            with open(curFile, 'r', encoding="utf8", newline='') as f:
              reader = csv.DictReader(f)
              for row in reader:
                # when searched string will be in title or in description
                if (d.Value.upper() in row["title"].upper()) or (d.Value.upper() in row["description"].upper()):
                  # and when the message is not deleted
                  if row["deleted"] == "0":
                    artResultDict[row["title"]] = row["uid"]
          if len(artResultDict) > 0:
            chfile = os.path.splitext(file)[0]
            self.searchResults[chfile] = artResultDict
      if len(self.searchResults) == 0:
        # Translators: Message of dialog saying that searched text was not found in any of the messages
        dlgMessage = _("%s not found in any message.") % d.Value
        # Translators: Title of warning dialog 
        dlgTitle = _("Warning")
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        return
      searchFilter = True
      searchChannelsList = []
      for key, value in self.searchResults.items():
        searchChannelsList.append(key)
      searchChannelsList.sort(key=locale.strxfrm)
      searchChannelsList.insert(0, self.displayAllChannelsTitle)
      self.channelsList.insertList(searchChannelsList)
      self.channelsList.selectItem(0)
      self.onchannelsListChange(None)
      self.channelsList.SetFocus()

  def exportChannelsToOpmlItem(self, evt):
    # translators: Title of save as dialog
    filename = wx.FileSelector(
      _("Save as"), default_filename="rss.opml", flags=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, parent=self
    )
    if filename:
      with open(filename, 'wb') as f:
        f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
      self.channelsList.SetFocus()

  def importChannelsFromOpmlItem(self, evt):
    # Translators: Title of dialog for import from OPML file
    with wx.FileDialog(
      self, _("&Import feeds from OPML file"),
      # Translators: OPML files as an option in the file type combo
      wildcard=_("OPML files (*.opml)|*.opml"),
      style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
    ) as importDialog:
      if importDialog.ShowModal() == wx.ID_CANCEL:
        return
      pathName = importDialog.GetPath()
    try:
      importOpmlTree = ET.parse(pathName)
    except Exception as e:
      raise e
    feeds = importOpmlTree.getroot().findall("./body/outline")
    body = self.opmlTree.getroot().find("body")
    chList = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    for feed in feeds:
      if not feed.get("xmlUrl"):
        # probably older format OPML than 2.0
        for subfeed in feed.iter("outline"):
          if subfeed.get("xmlUrl"):
            # when channel name is specified
            if subfeed.get("text"):
              # and is not stored in title attribute
              if not subfeed.get("title"):
                # we set the same name for title as it is in the text attribute
                subfeed.set("title", subfeed.get("text"))
              chname = subfeed.get("title")
              # check for channel name duplicity
              if chname in chList:
                # Translators: Added channel is already in the list of channels
                dlgMessage = _("The feed %s is already in the feed list.") % chname
                # Translators: Title of warning dialog
                dlgTitle = _("Warning")
                wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
              else:
                body.append(subfeed)
            else:
              # when the name is not present, we create channel as it is manually inserted (with existence check)
              self.createChannel(subfeed.get("xmlUrl"))
      else:
        # OPML version 2.0
        chname = feed.get("title")
        # check for channel name duplicity
        if chname in chList:
          # Translators: Added channel is already in the list of channels
          dlgMessage = _("The feed %s is already in the feed list.") % chname
          # Translators: Title of warning dialog
          dlgTitle = _("Warning")
          wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        else:
          body.append(feed)
    with open(opmlFilePath, 'wb') as f:
      f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.insertList(self.choices)
    self.addArchiveToList()
    self.channelsList.selectItem(self.channelsList.GetItemCount() - 1)
    self.onchannelsListChange(None)
    self.channelsList.SetFocus()
    if self.channelsList.GetItemCount() > 0:
      self.channelsList.selectItem(0)
    # switch off filter of channels with news
    self.filteredNews = False
    self.newsMenuItem.Check(False)

  def displayChannelsFilterItem(self, evt):
    if self.inArchive:
      return
    if self.filteredNews:
      # switch filter off, display all channels
      self.filteredNews = False
      # obtain channels from OPML
      self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
      self.choices.sort(key=locale.strxfrm)
      # assign channels to list items
      self.channelsList.insertList(self.choices)
      self.addArchiveToList()
      self.channelsList.selectItem(0)
      self.onchannelsListChange(None)
      self.channelsList.SetFocus()
    else:
      # display only channels with new messages
      global inProcessOfReadingNewArticles
      if inProcessOfReadingNewArticles:
        self.newsMenuItem.Check(False)
        ui.message(inprocessMessage)
        return
      if len(self.newsChannels) == 0:
        self.newsMenuItem.Check(False)
        # Translators: No channels with unread messages
        ui.message(_("No feeds with unread messages."))
      else:
        self.filteredNews = True
        newChannelsList = []
        for key, value in self.newsChannels.items():
          newChannelsList.append(key)  
        # newChannelsList = list(self.newsChannels.keys()), 0
        newChannelsList.sort(key=locale.strxfrm)
        self.channelsList.insertList(newChannelsList)
        self.channelsList.selectItem(0)
        self.onchannelsListChange(None)
        self.channelsList.SetFocus()

  def exitItem(self, evt):
    if self.taskbarIcon:
      try:
        self.taskbarIcon.Destroy()
      except:
        pass
    wx.CallAfter(self.Destroy)
    RssReader._instance = None

  def helpItem(self, evt):
    from . import addonDoc
    helpFile = addonDoc.getDocPath("readme.html")
    if helpFile:
      os.startfile(helpFile)
    else:
      os.startfile(os.path.join(_addonDir, "doc", "cs", "readme.html"))

  def openArchiveItem(self, evt):
    if self.channelsList.getItem(self.channelsList.GetItemCount()-1) != self.archiveTitle:
      return
    self.inArchive = True
    self.choices = []
    for filename in os.listdir(archiveFolder):
      self.choices.append(filename.rsplit(" ", 1)[0])
    self.channelsList.insertList(self.choices)
    self.channelsList.selectItem(0)
    self.channelsList.SetFocus()

  def aboutItem(self, evt):
    # Translators: version in about dialog
    dlgMessage = mainTitle + "\n" + _("version") + " " + _addonVersion + "\nCopyright (c) 2023-2025 Marek Susčík"
    # Translators: Title of About dialog
    dlgTitle = _("About")
    wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)

  def settingsForChannelItem(self, evt):
    selItem = self.channelsList.getSelectedItem()
    # some items are not channels, so they have no settings
    if (selItem == self.archiveTitle) or (selItem == self.displayAllChannelsTitle) or (selItem == self.displayAllMessagesTitle):
      return
    # channels in archive have no settings
    if self.inArchive:
      return
    ChannelSettingsDialog.channel = RssChannel()
    ChannelSettingsDialog.channel.setChannelName(selItem)
    self.Hide()
    self.Disable()
    try:
      ChannelSettingsDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def settingsItem(self, evt):
    self.Hide()
    self.Disable()
    try:
      RssReaderSettingsDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def backFromArchiveItem(self, evt):
    if not self.inArchive:
      return
    self.inArchive = False
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.insertList(self.choices)
    self.addArchiveToList()
    self.channelsList.selectItem(self.channelsList.GetItemCount() - 1)
    self.channelsList.SetFocus()

  def addArchiveToList(self):
    if os.path.exists(archiveFolder):
      self.channelsList.addItem(self.channelsList.GetItemCount(), self.archiveTitle)
    if self.channelsList.GetItemCount() >1:
      self.channelsList.addItem(0, self.displayAllMessagesTitle)
    if self.channelsList.GetItemCount() >0:
      self.channelsList.selectItem(0)

  def noChannels(self):
    if gui.messageBox(
      # Translators: Information about no channels in the reader and question if user can add channel now
      _("You don't have any feeds in your RSS reader. Do you want to add a feed now?"),
      # Translators: Title of wanring dialog
      _("Warning"),
      wx.YES | wx.NO | wx.ICON_QUESTION, self
    ) == wx.YES:
      if gui.messageBox(
        # Translators: Question whether user wants to choose channel from prepared channel in database
        _("Do you want to select a channel from a database of ready-made channels?"),
        # Translators: Title of warning dialog
        _("Warning"),
        wx.YES | wx.NO | wx.ICON_QUESTION, self
      ) == wx.YES:
        ChannelDatabaseDialog.passkey = True
        self.channelFromDatabaseItem(None)
      else:
        self.newChannelItem(None)

  def getCurrentChannel(self):
    return self.channel

  def playProgress(self, countProg, allProg):
    curProg = countProg / allProg * 1000
    lastProg = -1
    if lastProg != curProg:
      lastProg = curProg
      tones.beep(curProg, 40)

  def loadOpmlChannels(self):
    try:
      self.opmlTree = ET.parse(opmlFilePath)
    except Exception as e:
      raise e

  def saveOpmlChannel(self, title, url, type):
    element = ET.Element("outline")
    element.set("title", title)
    element.set("text", title)
    element.set("xmlUrl", url)
    element.set("type", type)
    body = self.opmlTree.getroot().find("body")
    body.append(element)
    with open(opmlFilePath, 'wb') as f:
      f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))

  def createChannel(self, address):
    cherror = False
    try:
      channel = RssChannel()
      channel.setUrl(address)
      if channel.readArticles(storeAudio=False) == -1:
        # Translators: Problem during channel loading
        raise ValueError(_("Problem loading the channel."))
    except Exception as e:
      wx.CallAfter(
        gui.messageBox,
        # Translators: channel from address can not be added
        _("Can not add feed from address %s: %s") % (address, e),
        # Translators: error
        (_("Error")),
        wx.OK | wx.ICON_ERROR
      )
      cherror = True
    if cherror:
      return None
    chName = channel.getChannelName()
    chType = channel.getFeedType()
    chList = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    if chName in chList:
      wx.CallAfter(
        gui.messageBox,
        # Translators: The channel is already in the list of channels
        _("The feed %s is already in the feed list.") % chName,
        # Translators: Title of warning dialog
        (_("Warning")),
        wx.OK | wx.ICON_ERROR
      )
      return ""
    self.saveOpmlChannel(chName, address, chType)
    return chName

  def channelFromDatabaseItem(self, evt):
    if self.filteredNews:
      # Translators: Dialog message that in this moment only the channels with new messages are displayed, for adding new channels please display all channels
      dlgMessage = _("There are only shown feeds with new messages. To be able to add feeds, please view all feeds.")
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    self.dbSelectedUrl = ""
    self.Hide()
    self.Disable()
    try:
      ChannelDatabaseDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if not self.dbSelectedUrl:
      return
    name = self.createChannel(self.dbSelectedUrl)
    if name:
      self.choices.append(name)
      self.choices.sort(key=locale.strxfrm)
      self.channelsList.insertList(self.choices)
      self.addArchiveToList()
      chIndex = self.channelsList.findString(name)
      self.channelsList.selectItem(chIndex)
      self.channelsList.SetFocus()

  def newChannelItem(self, evt):
    if self.filteredNews:
      # Translators: Dialog message that in this moment only the channels with new messages are displayed, for adding new channels please display all channels
      dlgMessage = _("There are only shown feeds with new messages. To be able to add feeds, please view all feeds.")
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    with wx.TextEntryDialog(
      # Translators: message to enter the address of new channel
      self, _("Enter the new feed address:"),
      # Translators: Title of new feed dialog
      _("New feed")
    ) as d:
      if d.ShowModal() == wx.ID_CANCEL or not d.Value:
        self.channelsList.SetFocus()
        return
    name = self.createChannel(d.Value)
    if name:
      self.choices.append(name)
      self.choices.sort(key=locale.strxfrm)
      self.channelsList.insertList(self.choices)
      self.addArchiveToList()
      chIndex = self.channelsList.findString(name)
      self.channelsList.selectItem(chIndex)
      self.channelsList.SetFocus()

  def openArticlesListItem(self, evt):
    self.onchannelsListChoice(self)

  def channelPropertiesItem(self, evt):
    try:
      result = ChannelPropertiesDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if result:
      # obtain channels from OPML
      self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
      self.choices.sort(key=locale.strxfrm)
      # assign channels to list items
      self.channelsList.insertList(self.choices)
      chIndex = self.channelsList.findString(self.newChannelName)
      self.channelsList.selectItem(chIndex)
      self.channelsList.SetFocus()

  def deleteChannelItem(self, evt):
    # when list is empty, do nothing
    if self.channelsList.GetItemCount() == 0:
      return
    # some items can not be deleted
    if (self.channelsList.getSelectedItem() == self.archiveTitle) or (self.channelsList.getSelectedItem() == self.displayAllChannelsTitle) or (self.channelsList.getSelectedItem() == self.displayAllMessagesTitle):
      return
    # when loading new messages, delete is not possible
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      dlgMessage = inprocessMessage
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    # when only channels with new messages are displayed, delete is not possible
    if self.filteredNews:
      # Translators: message that user has displayed only channels with new messages and that for possibility to delete channels it is necessary to display all channels
      dlgMessage = _("There are only shown feeds with new messages. To be able to delete feeds, please view all feeds.")
      # Translators: Title of the warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    if gui.messageBox(
      # Translators: question whether user wants to delete channel
      _("Do you want to delete feed %s?") % self.channelsList.getSelectedItem(),
      # Translators: dialog title removing channel
      _("Feed deletion"),
      wx.YES | wx.NO | wx.ICON_QUESTION, self
    ) == wx.NO:
      self.channelsList.SetFocus()
      return
    self.reallyDeleteChannel()

  def reallyDeleteChannel(self):
    delItem = self.channelsList.getSelectedItem()
    selItemNumber = self.channelsList.GetFirstSelected()
    if not self.inArchive:
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == delItem:
          body = self.opmlTree.getroot().find("body")
          body.remove(outline)
          break
      with open(opmlFilePath, 'wb') as f:
        f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # remove file with channel articles
    if self.inArchive:
      # find file to delete
      for filename in os.listdir(archiveFolder):
        if filename.startswith(delItem + " ("):
          break
      articleFile = os.path.join(archiveFolder, filename)
    else:
      articleFile = os.path.join(articlesFolder, delItem + ".txt")
    if os.path.isfile(articleFile):
      os.remove(articleFile)
    # delete channel settings file
    if not self.inArchive:
      articleFile = os.path.join(articlesFolder, delItem + ".json")
      if os.path.isfile(articleFile):
        os.remove(articleFile)
    # when deleted channel was the last one
    if self.channelsList.GetItemCount() == 1:
      self.channelsList.clearAllItems()
      if self.inArchive:
        # delete archive folder
        import shutil
        shutil.rmtree(archiveFolder)
        # Translators: message that the archive is empty
        dlgMessage = _("The archive is empty.")
        # Translators: title of the information dialog
        dlgTitle = _("Information")
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        wx.CallAfter(self.backFromArchiveItem, None)
      else:
        self.noChannels()
      return
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.insertList(self.choices)
    self.addArchiveToList()
    if selItemNumber == self.channelsList.GetItemCount():
      self.channelsList.selectItem(selItemNumber-1)
    else:
      self.channelsList.selectItem(selItemNumber)
    self.channelsList.SetFocus()

  def startreadingSelectedNewArticles(self):
    # Translators: message saying loading messages
    speech.speakMessage(_("loading messages."))
    selItem = self.channelsList.getSelectedItem()
    address = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        address = outline.get("xmlUrl")
        break
    if address:
      self.channel = RssChannel()
      self.channel.setUrl(address)
      self.channel.setChannelName(selItem)
      newArticlesCount = self.channel.readArticles()
      if newArticlesCount != -1:
        # add channel to newsChannels
        if newArticlesCount > 0:
          if selItem in self.newsChannels:
            self.newsChannels[selItem] = self.newsChannels[selItem] + newArticlesCount
          else:
            self.newsChannels[selItem] = newArticlesCount
        self.channel.writeArticles()
      # when uid of some articles is missing, writeArticles method will repair it
      if self.channel.missingUid:
        self.channel.writeArticles()
    if newArticlesCount <= 0:
      # Translators: message saying No new messages
      speech.speakMessage(_("No new messages."))
    else:
      playSound("news.wav")
      # Translators: message saying New messages
      speech.speakMessage(_("New messages %s") % str(newArticlesCount))
    global inProcessOfReadingNewArticles
    inProcessOfReadingNewArticles = False

  def readSelectedNewArticlesItem(self, evt):
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      ui.message(inprocessMessage)
      return
    if self.inArchive:
      # Translators: You are in archive. To receive new messages, first exit the archive using the Backspace key.
      ui.message(_("You're in the archive. To receive new messages, first exit the archive using the Backspace key."))
      return
    selItem = self.channelsList.getSelectedItem()
    # some items are not channels so messages cannot be retrieved
    if (selItem == self.archiveTitle) or (selItem == self.displayAllChannelsTitle) or (selItem == self.displayAllMessagesTitle):
      return
    # read selected new articles in separate thread, without blocking main program
    inProcessOfReadingNewArticles = True
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.readSelectedNewArticlesItem.__qualname__}",
      target=self.startreadingSelectedNewArticles
    )
    t.setDaemon(True)
    t.start()

  def startreadingAllNewArticles(self):
    # when only channels with new messages are displayed, it is necessary to switch to all channels
    if self.filteredNews:
      self.displayChannelsFilterItem(None)
      self.newsMenuItem.Check(False)
    # when search results are displayed, it is necessary to display all channels
    if searchFilter:
      self.clearSearch()
    newArticlesCount = 0
    if not self.taskbarIcon and not self.quietMode:
      # Translators: message saying Loading messages
      speech.speakMessage(_("loading messages."))
    progressCount = 1
    for selItem in self.channelsList.getAllItems():
      if selItem == self.archiveTitle:
        continue
      curArticlesCount = 0
      if not self.taskbarIcon and not self.quietMode:
        self.playProgress(progressCount, self.channelsList.GetItemCount())
      progressCount = progressCount + 1
      address = ""
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == selItem:
          address = outline.get("xmlUrl")
          break
      if address:
        self.readingChannel = RssChannel()
        self.readingChannel.setUrl(address)
        self.readingChannel.setChannelName(selItem)
        # when in tray or in quiet mode, reading new articles is quiet, without detecting errors
        if self.taskbarIcon or self.quietMode:
          curArticlesCount = self.readingChannel.readArticles(quiet=True)
        else:
          curArticlesCount = self.readingChannel.readArticles()
        if curArticlesCount != -1:
          newArticlesCount = newArticlesCount + curArticlesCount
          # add channel to newsChannels
          if curArticlesCount > 0:
            if selItem in self.newsChannels:
              self.newsChannels[selItem] = self.newsChannels[selItem] + curArticlesCount
            else:
              self.newsChannels[selItem] = curArticlesCount
          self.readingChannel.writeArticles()
        # when uid of some articles is missing, writeArticles method will repair it
        if self.readingChannel.missingUid:
          self.readingChannel.writeArticles()
    if newArticlesCount == 0:
      if not self.taskbarIcon and not self.quietMode:
        # Translators: message saying No new messages
        speech.speakMessage(_("No new messages."))
    else:
      if not self.taskbarIcon and not self.quietMode:
        playSound("news.wav")
        # Translators: message saying New messages
        speech.speakMessage(_("New messages %s") % str(newArticlesCount))
      global trayNewArticlesCount
      # store number of new messages to systray icon
      if self.taskbarIcon:
        trayNewArticlesCount += newArticlesCount
        # Translators: text in tray icon, when there are more than 4 new messages
        newsCountStr = _("new messages")
        if trayNewArticlesCount == 1:
          # Translators: text in tray icon, when there is only 1 new message
          newsCountStr = _("new message")
        elif 1 < (trayNewArticlesCount) < 5:
          # Translators: text in tray icon, when there are 2, 3, or 4 new messages
          newsCountStr = _("new messages")
        else:
          # Translators: text in tray icon, when there are more than 4 new messages
          newsCountStr = _("new messages")
        editTitle = mainTitle + ", " + str(trayNewArticlesCount) + " " + newsCountStr
        self.taskbarIcon.SetIcon(icon, editTitle)
    global inProcessOfReadingNewArticles
    inProcessOfReadingNewArticles = False
    if self.articlesWindow and self.articlesWindow.IsShown():
      wx.CallAfter(self.articlesWindow.refreshArticlesList)

  def readAllNewArticlesItem(self, evt):
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      speech.speakMessage(inprocessMessage)
      return
    if self.inArchive:
      # Translators: You are in archive. To receive new messages, first exit the archive using the Backspace key.
      ui.message(_("You're in the archive. To receive new messages, first exit the archive using the Backspace key."))
      return
    global searchFilter
    if not self.IsEnabled():
      # when articles are displayed and timer occurs, it is necessary to save actual state
      # partially copy of backtochannel method which is not possible to call directly from here
      if self.inArchive:
        self.channel.writeArticles(True)
      else:
        self.channel.writeArticles()
    # read all new articles in separate thread, without blocking main program
    inProcessOfReadingNewArticles = True
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.readAllNewArticlesItem.__qualname__}",
      target=self.startreadingAllNewArticles
    )
    t.setDaemon(True)
    t.start()
      
  def countArticles(self, chname):
    channel = RssChannel()
    channel.setChannelName(chname)
    fType = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == chname:
        fType = outline.get("type")
        break
    channel.setFeedType(fType)
    if self.inArchive:
      channel.loadArticlesFromFile(True)
    else:
      channel.loadArticlesFromFile()
    artNumber = channel.getNumberOfArticles()
    artCount = 0
    for x in range(artNumber):
      if channel.getArticleDeleted(x) == "0":
        artCount = artCount + 1
    return artCount

  def sayArticlesNumberItem(self, evt):
    sayMessage = ""
    selItem = self.channelsList.getSelectedItem()
    # some items are not channels so number of messages cannot be retrieved
    if (selItem == self.archiveTitle) or (selItem == self.displayAllChannelsTitle):
      return
    # find number of all messages in all channels
    if selItem == self.displayAllMessagesTitle:
      artCount = 0
      for file in os.listdir(articlesFolder):
        chfile = os.path.splitext(file)[0]
        artCount = artCount + self.countArticles(chfile)
    else:
      # find number of new messages
      newMessages = 0
      if not self.inArchive:
        if selItem in self.newsChannels:
          newMessages = self.newsChannels[selItem]
          # Translators: message saying New messages when user press F2 to find number of messages in channel
          sayMessage = _("New messages %s, ") % (str(newMessages))
      # find number of all messages
      artCount = self.countArticles(selItem)
    # Translators: message Number of messages when user press F2 to find number of messages in channel
    sayMessage = sayMessage + _("Number of messages %s") % (str(artCount))
    ui.message(sayMessage)

  def onchannelsListChange(self, evt):
    global currentChannelNewMessages
    global currentChannelResults
    global searchFilter
    if self.inArchive:
      return
    currentChannelNewMessages = ""
    currentChannelResults = ""
    selItem = self.channelsList.getSelectedItem()
    if selItem in self.newsChannels:
      currentChannelNewMessages = str(self.newsChannels[selItem])
    if selItem in self.searchResults:
      currentChannelResults = str(len(self.searchResults[selItem]))

  def clearSearch(self):
    global searchFilter
    searchFilter = False
    self.searchResults.clear()
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.insertList(self.choices)
    self.addArchiveToList()
    self.channelsList.selectItem(0)
    self.onchannelsListChange(None)
    self.channelsList.SetFocus()

  def displayAllArticles(self):
    global inAllArticles
    # start playing clock sound
    playSound("Clock.wav", True)
    self.channel = RssChannel()
    # read articles from all channels
    for file in os.listdir(articlesFolder):
      chfile = os.path.splitext(file)[0]
      self.channel.setChannelName(chfile)
      # find channel type
      foundinChannels = False
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == chfile:
          fType = outline.get("type")
          foundinChannels = True
          break
      if not foundinChannels:
        # when file in articles folder is not in the list of channels, we will delete this file due to async problem
        os.remove(os.path.join(articlesFolder, file))
        playSound("error.wav")
        playSound("Clock.wav", True)
        log.info("Due to async remove " + file)
      else:
        # when file is in the list of channels, everything is ok and we continue
        self.channel.setFeedType(fType)
        self.channel.loadArticlesFromFile()
    # Translators: this will be displayed as part of window title when messages from all channels are displayed
    self.channel.setChannelName(_("Display all feeds"))
    # delete number of new messages in all channels
    self.newsChannels.clear()
    # sort articles by date
    self.channel.articles.sort(key=self.channel.getArticleTimestamp, reverse=True)
    inAllArticles = True
    self.Hide()
    self.Disable()
    try:
      self.articlesWindow = ArticlesWindow(self)
      self.articlesWindow.Show()
      # ArticlesWindow(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def showArticlesForm(self):
    self.Hide()
    self.Disable()
    try:
      self.articlesWindow = ArticlesWindow(self)
      self.articlesWindow.Show()
      # ArticlesWindow(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def startreadingArticlesOnChannelsChoice(self):
    selItem = self.channelsList.getSelectedItem()
    fType = ""
    address = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        address = outline.get("xmlUrl")
        fType = outline.get("type")
        break
    if address:
      self.channel = RssChannel()
      self.channel.setUrl(address)
      self.channel.setChannelName(selItem)
      self.channel.setFeedType(fType)
      # when automatic downloading of new articles after opening channel is on and search filter is off and not in process of reading new articles
      global inProcessOfReadingNewArticles
      if newArticlesAfterOpening and not searchFilter and not inProcessOfReadingNewArticles:
        newArticlesCount = self.channel.readArticles()
        # when error occurs during reading new articles from Internet
        if newArticlesCount == -1:
          # behaviour is the same as reading new articles after opening channel is off
          self.channel.loadArticlesFromFile()
        else:
          # otherwise write new articles to the file
          self.channel.writeArticles()
          if newArticlesCount > 0:
            # play sound of new articles
            playSound("news.wav")
      else:
        # when automatic downloading of new articles after opening channel is off
        self.channel.loadArticlesFromFile()
      # when uid of some articles is missing, writeArticles method will repair it
      if self.channel.missingUid:
        self.channel.writeArticles()
      wx.CallAfter(self.showArticlesForm)

  def onchannelsListChoice(self, evt):
    global searchFilter
    selItem = self.channelsList.getSelectedItem()
    if (selItem == self.displayAllMessagesTitle) and (self.channelsList.GetFirstSelected() == 0):
      self.displayAllArticles()
    if (selItem == self.displayAllChannelsTitle) and (searchFilter):
      self.clearSearch()
      return
    if selItem == self.archiveTitle:
      self.openArchiveItem(None)
      return
    if self.inArchive:
      self.channel = RssChannel()
      self.channel.setChannelName(selItem)
      # find feed type in filename
      for filename in os.listdir(archiveFolder):
        if filename.startswith(selItem + " ("):
          break
      fType = filename.rsplit("(", 1)[1]
      fType = fType.split(")")[0]
      self.channel.setFeedType(fType)
      self.channel.loadArticlesFromFile(True)
      self.Hide()
      self.Disable()
      try:
        self.articlesWindow = ArticlesWindow(self)
        self.articlesWindow.Show()
        # ArticlesWindow(self).Show()
      except Exception as e:
        self.Enable()
        raise e
      return
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.onchannelsListChoice.__qualname__}",
      target=self.startreadingArticlesOnChannelsChoice
    )
    t.setDaemon(True)
    t.start()
        
  def onchannelsListKeyDown(self, evt):
    key = evt.GetKeyCode()
    evt.Skip()

  def onClose(self, evt):
    global downloadArticlesInterval
    if downloadArticlesInterval == 0:
      # when no automatic downloading of messages is set, app will close
      self.exitItem(None)
    else:
      # if any interval of downloading new messages is set, the app will go to tray
      self.Hide()
      if not self.taskbarIcon:
        self.taskbarIcon = RssTaskBarIcon(self)
        self.taskbarIcon.SetIcon(icon, mainTitle)

  def Show(self, show=True):
    # rewrite show method to delete tray icon when windows is displayed
    super(RssReader, self).Show(show)
    if show and self.taskbarIcon:
      # set number of new messages for tray to zero
      global trayNewArticlesCount
      trayNewArticlesCount = 0
      # delete tray icon
      try:
        self.taskbarIcon.Destroy()
      except:
        pass
      self.taskbarIcon = None


class RssChannel(object):
  
  def __init__(self):
    super(RssChannel, self).__init__()
    self.channelName = ""
    self.feedType = ""
    self.url = ""
    self.numberofNewArticles = 0
    self.channelTree = None
    self.articles = []
    self.newArticles = []
    self.ns = ""
    self.saveAudioSetting = True
    self.prefixPodcastFileSetting = 0
    self.missingUid = False

  def setUrl(self, url):
    self.url = url
  
  def applyNs(self, tag, ns=None):
    if ns is None:
      return tag
    else:
      return f"{ns}{tag}"
  
  def getArticleTimestamp(self, article):
    locale.setlocale(locale.LC_TIME, "en")
    try:
      if self.getFeedType() == "rss":
        date = article.find(self.applyNs("pubDate", self.ns)).text
        timestamp = time.mktime(datetime.datetime.strptime(date, "%a, %d %b %Y %H:%M:%S %z").timetuple())
      elif self.getFeedType() == "atom":
        date = article.find(self.applyNs("updated", self.ns)).text
        timestamp = time.mktime(datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%z").timetuple())
      return timestamp
    except Exception:
      try:
        if self.getFeedType() == "rss":
          date = article.find(self.applyNs("pubDate", self.ns)).text
          timestamp = time.mktime(datetime.datetime.strptime(date, "%a, %d %b %Y %H:%M:%S %Z").timetuple())
        elif self.getFeedType() == "atom":
          date = article.find(self.applyNs("updated", self.ns)).text
          timestamp = time.mktime(datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%Z").timetuple())
        return timestamp
      except Exception:
        return 0

  def removeDeletedArticles(self):
    # delete articles which have been deleted and are not already broadcasted
    articlesNumber = self.getNumberOfArticles()
    index = 0
    while index < articlesNumber:
      if self.articles[index].find(self.applyNs("deleted", self.ns)).text == "1":
        arTitle = self.getArticleTitle(index)
        arDate = self.getArticleDate(index)
        deletedFound = False
        for x in range(len(self.newArticles)):
          newArDate = self.getNewArticleDate(x)
          newArTitle = self.getNewArticleTitle(x)
          if arDate == newArDate and arTitle == newArTitle:
            deletedFound = True
            break
        if not deletedFound:
          del self.articles[index]
          articlesNumber = articlesNumber - 1
          index = index -1
      index = index + 1
  
  def joinArticleSources(self, storeAudio=True, quiet=False):
    global saveAudioFromPodcasts
    # load channel settings
    self.loadSettings()
    # only new articles are added
    self.numberofNewArticles = 0
    for x in range(len(self.newArticles)):
      articleFound = False
      arDate = self.getNewArticleDate(x)
      arTitle = self.getNewArticleTitle(x)
      for y in range(self.getNumberOfArticles()):
        fiDate = self.getArticleDate(y)
        fiTitle = self.getArticleTitle(y)
        if arDate == fiDate and arTitle == fiTitle:
          articleFound = True
          break
      if not articleFound:
        addArticle = self.newArticles[x]
        # add deleted field
        delElem = ET.Element(self.applyNs("deleted", self.ns))
        delElem.text = "0"
        addArticle.append(delElem)
        # add audioposition field
        audposElem = ET.Element(self.applyNs("audioposition", self.ns))
        addArticle.append(audposElem)
        # add uid field
        uidElem = ET.Element(self.applyNs("uid", self.ns))
        uidElem.text = str(uuid.uuid4())
        addArticle.append(uidElem)
        self.articles.append(addArticle)
        self.numberofNewArticles += 1
        # when user sets saving audio from podcasts, we will continue to save audio data to disk
        if saveAudioFromPodcasts:
          # we store audio only from channels which user allows
          if self.saveAudioSetting:
            # we want to store audio files to disk now, we don't want it only when creating new channel
            if storeAudio:
              # store audio only when article has audio url
              if self.getAudioUrl(x):
                # save audio from the last added new article
                self.saveAudio(-1, progress=False, quiet=quiet)

  def loadArticlesFromFile(self, fromArchive=False, daysspan=0):
    channelName = self.getChannelName()
    feedType = self.getFeedType()
    if fromArchive:
      csvFileName = channelName + " (" + feedType + ")" + ".txt"
      articlesFile = os.path.join(archiveFolder, csvFileName)
    else:
      csvFileName = channelName + ".txt"
      articlesFile = os.path.join(articlesFolder, csvFileName)
    if not os.path.isfile(articlesFile):
      return False
    with open(articlesFile, 'r', encoding="utf8", newline='') as f:
      reader = csv.DictReader(f)
      self.missingUid = False
      try:
        for row in reader:
          if feedType == "atom":
            artElem = ET.Element(self.applyNs("entry", self.ns))
          else:
            artElem = ET.Element(self.applyNs("item", self.ns))
          title = ET.Element(self.applyNs("title", self.ns))
          title.text = row['title']
          artElem.append(title)
          if feedType == "atom":
            desc = ET.Element(self.applyNs("summary", self.ns))
          else:
            desc = ET.Element(self.applyNs("description", self.ns))
          desc.text = row['description']
          artElem.append(desc)
          if feedType == "atom":
            adate = ET.Element(self.applyNs("updated", self.ns))
          else:
            adate = ET.Element(self.applyNs("pubDate", self.ns))
          adate.text = row['date']
          artElem.append(adate)
          link = ET.Element(self.applyNs("link", self.ns))
          if feedType == "atom":
            link.set("href", row['link'])
          else:
            link.text = row['link']
          artElem.append(link)
          deleted = ET.Element(self.applyNs("deleted", self.ns))
          deleted.text = row['deleted']
          artElem.append(deleted)
          aurl = ET.Element(self.applyNs("enclosure", self.ns))
          try:
            aurl.set("url", row['audiourl'])
          except:
            aurl.set("url", "")
          try:
            if row['audiourl']:
              aurl.set("type", "audio/mpeg")
            else:
              aurl.set("type", "")
          except:
            aurl.set("type", "")
          artElem.append(aurl)
          apos = ET.Element(self.applyNs("audioposition", self.ns))
          try:
            if row['audiopos']:
              apos.text = row['audiopos']
            else:
              apos.text = ""
          except:
            apos.text = ""
          artElem.append(apos)
          channel = ET.Element(self.applyNs("channel", self.ns))
          channel.text = channelName
          artElem.append(channel)
          feed = ET.Element(self.applyNs("feedtype", self.ns))
          feed.text = feedType
          artElem.append(feed)
          uid = ET.Element(self.applyNs("uid", self.ns))
          try:
            if row['uid']:
              uid.text = row['uid']
            else:
              uid.text = str(uuid.uuid4())
          except:
            self.missingUid = True
            uid.text = str(uuid.uuid4())
          artElem.append(uid)
          if daysspan == 0:
            self.articles.append(artElem)
          else:
            articleTimestamp = self.getArticleTimestamp(artElem)
            articleDatetime = datetime.datetime.fromtimestamp(articleTimestamp)
            currentDatetime = datetime.datetime.now()
            diffDatetime = currentDatetime - articleDatetime
            diffDays = diffDatetime.days
            if diffDays <= daysspan:
              self.articles.append(artElem)
      except Exception as e:
        playSound("error.wav")
        # Translators: Error during opening channel
        speech.speakMessage(_("Error when opening a feed %s: %s") % (channelName, e))
    return True

  def readArticles(self, quiet=False, storeAudio=True):
    try:
      response = urllib.request.urlopen(self.url)
    except Exception:
      userAgent = "UniversalFeedParser/5.0.1 +http://feedparser.org/"
      headers = {'User-Agent': userAgent}
      req = urllib.request.Request(self.url, None, headers)
      try:
        response = urllib.request.urlopen(req)
      except Exception as e:
        if not quiet:
          playSound("error.wav")
          # Translators: Error during loading channel
          speech.speakMessage(_("Error when loading a feed %s: %s") % (self.channelName, e))
        return -1
    try:
      xmlstr = response.read().strip()
      self.channelTree = ET.fromstring(xmlstr)
    except Exception as e:
      if not quiet:
        playSound("error.wav")
        # Translators: Error during loading channel
        speech.speakMessage(_("Error when loading a feed %s: %s") % (self.channelName, e))
      return -1
    tag = self.channelTree.tag
    self.ns = "%s}" % tag.split("}", 1)[0] if "}" in tag else None
    # rss or atom
    if tag.endswith("rss"):
      self.main = self.channelTree.find(self.applyNs("channel", self.ns))
      self.newArticles = self.main.findall(self.applyNs("item", self.ns))
      self.feedType = "rss"
    elif tag.endswith("feed"):
      self.main = self.channelTree
      self.newArticles = self.main.findall(self.applyNs("entry", self.ns))
      self.feedType = "atom"
    else:
      # Translators: only for log: unknown channel type
      log.debugWarning(_("Unknown RSS feed type"), exc_info=True)
      raise
    # remove HTML tags from title and description of all new articles
    for article in self.newArticles:
      # remove tags from title
      try:
        arTitleTr = article.find(self.applyNs("title", self.ns))
        soup = BeautifulSoup(arTitleTr.text)
        arTitleTr.text = soup.getText(strip=True)
      except: pass
      # remove tags from description
      try:
        if self.getFeedType() == "rss":
          arDescTr = article.find(self.applyNs("description", self.ns))
        elif self.getFeedType() == "atom":
          arDescTr = article.find(self.applyNs("summary", self.ns))
        soup = BeautifulSoup(arDescTr.text)
        arDescTr.text = soup.getText(strip=True)
      except: pass
    self.setChannelName(validFileName(self.getChannelName()))
    if self.loadArticlesFromFile():
      self.removeDeletedArticles()
    self.joinArticleSources(storeAudio=storeAudio, quiet=quiet)
    self.articles.sort(key=self.getArticleTimestamp, reverse=True)
    self.index = 0
    return self.numberofNewArticles

  def writeArticles(self, fromArchive=False):
    self.missingUid = False
    global articlesHeader
    channelName = self.getChannelName()
    feedType = self.getFeedType()
    if fromArchive:
      csvFileName = channelName + " (" + feedType + ")" + ".txt"
      articlesFile = os.path.join(archiveFolder, csvFileName)
    else:
      csvFileName = channelName + ".txt"
      articlesFile = os.path.join(articlesFolder, csvFileName)
    with open(articlesFile, 'w', encoding="utf8", newline='') as f:
      writer = csv.writer(f)
      # write the header
      writer.writerow(articlesHeader)
      # write articles
      for x in range(self.getNumberOfArticles()):
        arTitle = self.getArticleTitle(x)
        arDesc = self.getArticleDescription(x)
        # too long description have to be shortened
        if arDesc:
          if len(arDesc) > 130000:
            arDesc = arDesc[:130000] + " Příliš dlouhý popis, zkráceno."
        arDate = self.getArticleDate(x)
        arLink = self.getArticleLink(x)
        arDeleted = self.getArticleDeleted(x)
        arAudio = self.getAudioUrl(x)
        arAudpos = self.getAudioPosition(x)
        arUid = self.getArticleUid(x)
        if not arUid:
          arUid = str(uuid.uuid4())
        articleRow = [arTitle, arDesc, arDate, arLink, arDeleted, arAudio, arAudpos, arUid]
        writer.writerow(articleRow)

  def getChannelUrl(self):
    return self.url

  def getFeedType(self):
    return self.feedType

  def setFeedType(self, fType=""):
    self.feedType = fType

  def setChannelName(self, chName=""):
    self.channelName = chName

  def getChannelName(self):
    if self.channelName:
      return self.channelName
    else:
      try:
        self.setChannelName(self.main.find(self.applyNs("title", self.ns)).text.strip())
        return self.channelName
      except Exception:
        return ""

  def getChannelNamefromArticle(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("channel", self.ns)).text
    except:
      return ""

  def getFeedTypefromArticle(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("feedtype", self.ns)).text
    except:
      return ""

  def getArticleTitle(self, index=None):
    if index is None:
      index = self.index
    try:
      arTitle = self.articles[index].find(self.applyNs("title", self.ns)).text
    except Exception:
      # Translators: Can not get the message title
      arTitle = _("Unable to retrieve the message title.")
    if not arTitle:
      # Translators: No title
      arTitle = _("No title")
    return arTitle

  def getNewArticleTitle(self, index=None):
    if index is None:
      index = self.index
    try:
      arTitle = self.newArticles[index].find(self.applyNs("title", self.ns)).text
    except Exception:
      # Translators: Can not get the message title
      arTitle = _("Unable to retrieve the message title.")
    if not arTitle:
      arTitle = "Bez titulku"
    return arTitle

  def getAudioUrl(self, index=None):
    if index is None:
      index = self.index
    # if attribute type contains audio, continue otherwise return ""
    try:
      enclosureType = self.articles[index].find(self.applyNs("enclosure", self.ns)).get("type")
    except:
      return ""
    if (enclosureType) and ("audio" in enclosureType):
      # when audio is in type, return url
      try:
        return self.articles[index].find(self.applyNs("enclosure", self.ns)).get("url")
      except Exception:
        return ""
    else:
      # when audio is not in type, return ""
      return ""

  def getAudioPosition(self, index=None):
    if index is None:
      index = self.index
    try:
      arAudioPos = self.articles[index].find(self.applyNs("audioposition", self.ns)).text
    except:
      arAudioPos = ""
    return arAudioPos

  def getArticleLink(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        return self.articles[index].find(self.applyNs("link", self.ns)).text
      elif self.getFeedType() == "atom":
        return self.articles[index].find(self.applyNs("link", self.ns)).get("href")
    except Exception:
      return ""

  def getNewArticleLink(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        return self.newArticles[index].find(self.applyNs("link", self.ns)).text
      elif self.getFeedType() == "atom":
        return self.newArticles[index].find(self.applyNs("link", self.ns)).get("href")
    except Exception:
      return ""

  def getArticleDeleted(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("deleted", self.ns)).text
    except Exception:
      return None

  def getArticleUid(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("uid", self.ns)).text
    except Exception:
      return None

  def getArticleDescription(self, index=None):
    if index is None:
      index = self.index
    description = None
    try:
      if self.getChannelUrl().startswith("https://www.google.com") and "/alerts/" in self.getChannelUrl():
        description = self.articles[index].find(self.applyNs("content", self.ns)).text
      if description is not None:
        return description
      if self.getFeedType() == "rss":
        description = self.articles[index].find(self.applyNs("description", self.ns)).text
      elif self.getFeedType() == "atom":
        description = self.articles[index].find(self.applyNs("summary", self.ns)).text
      if description is None:
        description = self.articles[index].find(self.applyNs("content", self.ns)).text
      return description
    except Exception:
      return None

  def getNewArticleDate(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        date = self.newArticles[index].find(self.applyNs("pubDate", self.ns)).text
      elif self.getFeedType() == "atom":
        date = self.newArticles[index].find(self.applyNs("updated", self.ns)).text
      return date.strip()
    except Exception:
      return ""

  def getArticleDate(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        date = self.articles[index].find(self.applyNs("pubDate", self.ns)).text
      elif self.getFeedType() == "atom":
        date = self.articles[index].find(self.applyNs("updated", self.ns)).text
      return date.strip()
    except Exception:
      return ""

  def getNumberOfArticles(self):
    return len(self.articles)

  def markOlderArticlesForDeletion(self, days=0):
    if days == 0:
      return False
    global deleteAllAfter
    articlesNumber = self.getNumberOfArticles()
    index = 0
    wasDeleted = False
    while index < articlesNumber:
      diffDays = 0
      arDate = self.getArticleDate(index)
      arTitle = self.getArticleTitle(index)
      artElem = self.articles[index]
      articleTimestamp = self.getArticleTimestamp(artElem)
      if articleTimestamp:
        articleDatetime = datetime.datetime.fromtimestamp(articleTimestamp)
        currentDatetime = datetime.datetime.now()
        diffDatetime = currentDatetime - articleDatetime
        diffDays = diffDatetime.days
      if diffDays > days:
        wasDeleted = True
        # when all articles is displayed
        if inAllArticles:
          # find channel name and article title from currently selected article
          delChannelName = self.getChannelNamefromArticle(index)
          delTitle = self.getArticleTitle(index)
          # assign channel name as key to dict and add article title to list, value of this dict
          deleteAllAfter[delChannelName].append(delTitle)
        else:
          delMark = self.articles[index].find(self.applyNs("deleted", self.ns))
          delMark.text = "1"
      index += 1
    return wasDeleted

  def getPodcastFilePrefix(self, folder):
    maxNumber = 0
    filesToRename = []
    for file in os.listdir(folder):
      match = re.match(r'^(\d+)(.*)', file)
      if match:
        number = int(match.group(1))
        maxNumber = max(maxNumber, number)
        filesToRename.append((file, number, match.group(2) if match.group(2) else ""))
    formatLength = len(str(maxNumber + 1))
    if formatLength < 2:
      formatLength = 2
    if maxNumber >= 99:
      for file, number, restName in sorted(filesToRename, key=lambda x: x[1]):
        newName = f"{number:0{formatLength}d}{restName}"
        originalPath = os.path.join(folder, file)
        newPath = os.path.join(folder, newName)
        try:
          os.rename(originalPath, newPath)
        except OSError as e:
          log.error(f"Error renaming {file} to {newName}: {e}")
    return f"{maxNumber + 1:0{formatLength}d}"

  def saveAudio(self, index=None, force=False, progress=True, quiet=False):
    if index is None:
      index = self.index
    global podcastsMp3Path
    if not os.path.exists(podcastsMp3Path):
      os.mkdir(podcastsMp3Path)
    audioUrl = self.getAudioUrl(index)
    channelName = self.getChannelName()
    articleTitle = self.getArticleTitle(index)
    articleTitle = validFileName(articleTitle)
    articleDate = self.getArticleDate(index)
    artDateTime = getRightDateFormat(articleDate)
    datestr = ""
    if artDateTime:
      locale.setlocale(locale.LC_TIME, "czech")
      datestr = datetime.datetime.strftime(artDateTime, "%Y-%m-%d %H-%M-%S")
      datestr = datestr + " "
    targetFolder = os.path.join(podcastsMp3Path, channelName)
    if not os.path.exists(targetFolder):
      os.mkdir(targetFolder)
    match self.prefixPodcastFileSetting:
      case 0: # not set, main settings will be applied
        global prefixPodcastFile
        match prefixPodcastFile:
          case 1: # prefix: date and time
            storePath = os.path.join(targetFolder, datestr + articleTitle + ".mp3")
          case 2: # prefix: number
            storePath = os.path.join(targetFolder, str(self.getPodcastFilePrefix(targetFolder)) + " " + articleTitle + ".mp3")
          case _: # everything else, no prefix
            storePath = os.path.join(targetFolder, articleTitle + ".mp3")
      case 2: # prefix: date and time
        storePath = os.path.join(targetFolder, datestr + articleTitle + ".mp3")
      case 3: # prefix: number
        storePath = os.path.join(targetFolder, str(self.getPodcastFilePrefix(targetFolder)) + " " + articleTitle + ".mp3")
      case _: # everything else, no prefix
        storePath = os.path.join(targetFolder, articleTitle + ".mp3")
    if os.path.isfile(storePath):
      if force:
        os.remove(storePath)
      else:
        return storePath
    downloader = Downloader(downloadUrl=audioUrl, downloadTarget=storePath, quiet=quiet)
    if progress:
      downloader.startDownload()
    else:
      downloader.downloadWithoutProgress()
    return ""

  def getChannelSettingsFilePath(self):
    channelName = self.getChannelName()
    return os.path.join(articlesFolder, channelName + ".json")    

  def loadSettings(self):
    SetFile = self.getChannelSettingsFilePath()
    if os.path.isfile(SetFile):
      with open(SetFile, 'r') as f:
        config = json.load(f)
      try:
        self.saveAudioSetting = config['SaveAudio']
      except:
        self.saveAudioSetting = True
      try:
        self.prefixPodcastFileSetting = config['PrefixPodcastFile']
      except:
        self.prefixPodcastFileSetting = 0

  def saveSettings(self):
    config = {"SaveAudio": self.saveAudioSetting, "PrefixPodcastFile": self.prefixPodcastFileSetting}
    SetFile = self.getChannelSettingsFilePath()
    with open(SetFile, 'w') as f:
      json.dump(config, f)


class ArticlesWindow(wx.Frame):

  needSave = False
  lastSelArtItem = -1
  articleschoices = []
  
  def __init__(self, parent):
    # The window title
    # Translators: The messages window title
    super(ArticlesWindow, self).__init__(parent, size=(1000, 800), title=_("Messages - %s") % parent.channel.getChannelName())
    global articleHwnd
    articleHwnd = self.GetHandle()

    # icon
    self.SetIcon(icon)
    if not inAllArticles:
      newMessagesCount = 0
      newMessagesCount = parent.channel.numberofNewArticles
      if parent.channel.getChannelName() in parent.newsChannels:
        newMessagesCount = newMessagesCount + parent.newsChannels[parent.channel.getChannelName()]
      if newMessagesCount > 0:
        # Translators: The messages window title followed by a number of new messages
        self.SetTitle(_("Messages - ") + parent.channel.getChannelName() + " (" + str(newMessagesCount) + ")")

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    self.completion_event, self.EVT_COMPLETION_EVENT = wx.lib.newevent.NewCommandEvent()
    self.Bind(self.EVT_COMPLETION_EVENT, self.on_download_complete)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    # Translators: Display message menu item
    item = menu.Append(wx.ID_ANY, _("Display &message\tEnter"))
    self.Bind(wx.EVT_MENU, self.openArticleItem, item)
    # Translators: Display message in browser menu item
    item = menu.Append(wx.ID_ANY, _("Display message in &browser\tCtrl+Enter"))
    self.Bind(wx.EVT_MENU, self.openArticleInBrowserItem, item)
    # Translators: Display message as text menu item
    item = menu.Append(wx.ID_ANY, _("Display message as &text\tShift+Enter"))
    self.Bind(wx.EVT_MENU, self.openArticleAsTextItem, item)
    # Translators: Delete message menu item
    item = menu.Append(wx.ID_ANY, _("Delete m&essage\tDelete"))
    self.Bind(wx.EVT_MENU, self.DeleteArticleItem, item)
    # Translators: Announce message description menu item
    item = menu.Append(wx.ID_ANY, _("&Announce message description\tCtrl+F2"))
    self.Bind(wx.EVT_MENU, self.sayArticleDescriptionItem, item)
    # Translators: Announce date and position of message menu item
    item = menu.Append(wx.ID_ANY, _("Announce message date and &position\tF2"))
    self.Bind(wx.EVT_MENU, self.sayArticleDateItem, item)
    # Translators: Send message by mail menu item
    item = menu.Append(wx.ID_ANY, _("Send message by e-&mail\tCtrl+E"))
    self.Bind(wx.EVT_MENU, self.sendArticleItem, item)
    # Translators: Archive message menu item
    item = menu.Append(wx.ID_ANY, _("Ar&chive message\tF6"))
    self.Bind(wx.EVT_MENU, self.archiveArticleItem, item)
    # Translators: Select all menu item
    item = menu.Append(wx.ID_ANY, _("&Select all\tCtrl+A"))
    self.Bind(wx.EVT_MENU, self.selectAllItem, item)
    # Translators: Save audio file from selected messages menu item
    item = menu.Append(wx.ID_ANY, _("Save audio &file from selected messages\tCtrl+S"))
    self.Bind(wx.EVT_MENU, self.saveAudioFileFromSelectedArticles, item)
    # Translators: Back to feed list menu item
    item = menu.Append(wx.ID_ANY, _("&Back to feed list\tBackspace"))
    self.Bind(wx.EVT_MENU, self.backtoChannelsItem, item)
    menu.AppendSeparator()
    # Translators: Exit menu item
    item = menu.Append(wx.ID_EXIT, _("&Exit"))
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    # Translators: Title of the menu
    menuBar.Append(menu, _("Messages"))
    self.SetMenuBar(menuBar)
    
    # delete older articles
    if deleteOlderArticles > 0:
      delDays = convertDeleteOlderArticles(deleteOlderArticles)
      self.needSave = self.Parent.channel.markOlderArticlesForDeletion(delDays)

    # assign articles to listbox
    self.articlesData = self.getArticlesList()
    # Translators: Title of list column in messages window
    self.articlesList = VirtualListCtrlUid(self, _("Messages"), self.articlesData)
    self.articlesList.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.onarticlesListChange)
    self.articlesList.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onarticlesListChoice)
    self.articlesList.Bind(wx.EVT_LIST_KEY_DOWN, self.onarticlesListKeyDown)
    self.articlesList.SetMinSize((1000, 800))
    sHelper.addItem(self.articlesList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    # stop sound if all articles from all channels are displayed
    if inAllArticles:
      stopsound()
    
    self.articlesList.SetFocus()
    self.CentreOnScreen()
    if self.articlesList.GetItemCount() > 0:
      self.articlesList.selectItem(0)
      self.onarticlesListChange(None)
    else:
      self.noArticles()

  def refreshArticlesList(self):
    lastItemId = self.articlesList.getSelectedSmartItem()
    self.Parent.channel.articles.clear()
    self.Parent.channel.loadArticlesFromFile()
    self.articlesData = self.getArticlesList()
    self.articlesList.insertList(self.articlesData)
    newIndex = -1
    if lastItemId is not None:
      newIndex = self.articlesList.GetIndexFromItemId(lastItemId)
    if newIndex != -1:
      self.articlesList.selectItem(newIndex)
    else:
      self.articlesList.selectItem(0)

  def getArticlesList(self):
    articlesListWithId = []
    for index in range(self.Parent.channel.getNumberOfArticles()):
      articleDeleted = self.Parent.channel.getArticleDeleted(index)
      # get only articles which were not deleted
      if articleDeleted == "0":
        articleTitle = self.Parent.channel.getArticleTitle(index)
        articleId = self.Parent.channel.getArticleUid(index)
        if not articleTitle:
          # Translators: Can not get the message title
          articleTitle = _("Unable to retrieve the message title.")
        if articleId:
          articlesListWithId.append({"title": articleTitle, "id": articleId})
        else:
          log.info("This title has no id: " + articleTitle)
    # filter articles if search is active
    global searchFilter
    if searchFilter:    
      channelName = self.Parent.channel.getChannelName()
      if channelName in self.Parent.searchResults:
        searchResultIds = list(self.Parent.searchResults[channelName].values()) 
        articlesListWithId = [article for article in articlesListWithId if article['id'] in searchResultIds]
    return articlesListWithId

  def getArticleIndexById(self, articleId):
    for i in range(self.Parent.channel.getNumberOfArticles()):
      articleUid = self.Parent.channel.getArticleUid(i)
      if articleUid == articleId:
        return i
    return -1

  def getSelectedArticleIndex(self):
    selectedItemId = self.articlesList.getSelectedSmartItem()
    if selectedItemId is not None:
      articleIndex = self.getArticleIndexById(selectedItemId)
      return articleIndex
    else:
      return -1

  def noArticles(self):
    # Translators: Message after opening empty feed
    dlgMessage = _("There are no messages in this feed.")
    # Translators: Title of the information dialog
    dlgTitle = _("Information")
    wx.MessageBox(dlgMessage, dlgTitle)
    wx.CallAfter(self.backtoChannelsItem, None)

  def startDownloadingAudioFilesFromSelectedArticles(self):
    saveList = self.articlesList.getSelectedIndices()
    for itemIndex in saveList:
      itemIdToDownload = self.articlesList.GetItemIdFromIndex(itemIndex)
      foundArticleIndex = self.getArticleIndexById(itemIdToDownload)
      if foundArticleIndex != -1:
        if self.Parent.channel.getAudioUrl(foundArticleIndex):
          saveResult = self.Parent.channel.saveAudio(foundArticleIndex, force=False, progress=False, quiet=False)
    wx.PostEvent(self, wx.CommandEvent(self.EVT_COMPLETION_EVENT.typeId))

  def saveAudioFileFromSelectedArticles(self, evt):
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      ui.message(inprocessMessage)
    if self.articlesList.GetItemCount() == 0:
      return
    saveList = self.articlesList.getSelectedIndices()
    if len(saveList) == 0:
      return
    elif len(saveList) > 1:
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      # Translators: message whether user wants to save audio files from selected messages
      dlgMessage = _("Do you want to save audio files from %s selected messages?") % str(len(saveList))
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.NO == result:
        return
    # translators: message announcing the start of downloading selected audio podcasts
    ui.message(_("Downloading."))
    # download audio files in separate thread, without blocking main program
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.saveAudioFileFromSelectedArticles.__qualname__}",
      target=self.startDownloadingAudioFilesFromSelectedArticles
    )
    t.setDaemon(True)
    t.start()

  def on_download_complete(self, event):
    # Translators: Message that downloading of audio files was finished
    dlgMessage = _("Downloading of audio files was finished.")
    # Translators: Title of warning dialog
    dlgTitle = _("Warning")
    wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)

  def selectAllItem(self, evt):
    itemsCount = self.articlesList.GetItemCount()
    for index in range(itemsCount):
      self.articlesList.Select(index)
    # Translators: announce selection when messages are selected
    msg = ngettext("Selected", "Selected", itemsCount) 
    # Translators: word message in selected message announcement (e.g. Selected 1 message)
    msg = msg + " " + str(itemsCount) + " " + ngettext("message", "messages", itemsCount) + "."
    ui.message(msg)

  def DeleteArticleItem(self, evt, fromArchive=False):
    global deleteAllAfter
    global inProcessOfReadingNewArticles
    global inAllArticles
    if inProcessOfReadingNewArticles:
      dlgMessage = inprocessMessage
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    if self.articlesList.GetItemCount() == 0:
      return
    delList = self.articlesList.getSelectedIndices()
    if len(delList) == 0:
      return
    elif len(delList) == 1:
      delIndex = delList[0]
      delItemId = self.articlesList.GetItemIdFromIndex(delIndex)
      delItemTitle = self.articlesList.GetItemText(delIndex)
      # Translators: question whether user wants to delete message
      dlgMessage = _("Do you really want to delete message %s?") % delItemTitle
    else:
      # Translators: question whether user wants to delete more than one message (e.g. Do you really want to delete 5 messages?)
      dlgMessage = ngettext("Do you really want to delete %s message", "Do you really want to delete %s messages", len(delList)) % str(len(delList))
    if askArticleDeletion and not fromArchive:
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.YES != result:
        return
    self.articlesList.Freeze()
    for itemIndex in reversed(self.articlesList.getSelectedIndices()):
      itemIdToDelete = self.articlesList.GetItemIdFromIndex(itemIndex)
      foundArticleIndex = self.getArticleIndexById(itemIdToDelete)
      if foundArticleIndex != -1:
        # when all articles is displayed
        if inAllArticles:
          # find channel name from currently selected article
          delChannelNamelm = self.Parent.channel.articles[foundArticleIndex].find(self.Parent.channel.applyNs("channel", self.Parent.channel.ns))
          delChannelName = delChannelNamelm.text
          # assign channel name as key to dict and add article uid to list, value of this dict
          if delChannelName not in deleteAllAfter:
            deleteAllAfter[delChannelName] = []
          deleteAllAfter[delChannelName].append(itemIdToDelete)
        else:
          # when articles from one channel are displayed
          if self.Parent.inArchive:
            del self.Parent.channel.articles[foundArticleIndex]
          else:
            delMark = self.Parent.channel.articles[foundArticleIndex].find(self.Parent.channel.applyNs("deleted", self.Parent.channel.ns))
            if delMark is not None:
              delMark.text = "1"
        self.needSave = True
        selItemNumber = itemIndex
        if selItemNumber != -1:
          self.articlesList.deleteItem(selItemNumber)
          # delete article from search results list in searchresult dictionary
          if searchFilter:
            channelName = self.Parent.channel.getChannelName()
            if channelName in self.Parent.searchResults:
              channel_results = self.Parent.searchResults[channelName]
              title_to_remove = None
              for title, article_id in channel_results.items():
                if article_id == itemIdToDelete:
                  title_to_remove = title
                  break
              if title_to_remove:
                del self.Parent.searchResults[channelName][title_to_remove]
              else:
                log.info(f"DeleteArticleItem: Položka s ID '{itemIdToDelete}' nebyla nalezena v self.Parent.searchResults")
            else:
              log.info(f"DeleteArticleItem: Kanál '{channelName}' nebyl nalezen v self.Parent.searchResults")              
      else:
        log.info(f"DeleteArticleItem: Článek s itemIdToDelete={itemIdToDelete} nebyl nalezen.")
    self.articlesList.Thaw()
    self.articlesList.Refresh()
    if self.articlesList.GetItemCount() > 0:
      if selItemNumber == self.articlesList.GetItemCount():
        self.articlesList.selectItem(selItemNumber-1)
      else:
        self.articlesList.selectItem(selItemNumber)
      self.onarticlesListChange(None)
    else:
      self.noArticles()

  def archiveArticleItem(self, evt):
    archList = self.articlesList.getSelectedIndices()
    if len(archList) == 0:
      return
    elif len(archList) == 1:
      archIndex = archList[0]
      archItemId = self.articlesList.GetItemIdFromIndex(archIndex)
      archItemTitle = self.articlesList.GetItemText(archIndex)
      # Translators: question if user wants to move message to archive
      dlgMessage = _("Do you want to move to archive message %s?") % archItemTitle
    else:
      # Translators: question if user wants to move more than one message to archive
      dlgMessage = ngettext("Do you want to move to archive %s message?", "Do you want to move to archive %s messages?", len(archList)) % str(len(archList))
    if askArticleArchivation:
      # Translators: Title of warning dialog
      dlgTitle = _("Warning")
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.YES != result:
        return
    global articlesHeader
    if not os.path.exists(archiveFolder):
      os.mkdir(archiveFolder)
    articlesToArchiveIds = []
    for itemIndex in self.articlesList.getSelectedIndices():
      itemIdToArchive = self.articlesList.GetItemIdFromIndex(itemIndex)
      foundArticleIndex = self.getArticleIndexById(itemIdToArchive)
      if foundArticleIndex != -1:
        articleTitle = self.Parent.channel.getArticleTitle(foundArticleIndex)
        articleDesc = self.Parent.channel.getArticleDescription(foundArticleIndex)
        articleDate = self.Parent.channel.getArticleDate(foundArticleIndex)
        articleLink = self.Parent.channel.getArticleLink(foundArticleIndex)
        articleAudio = self.Parent.channel.getAudioUrl(foundArticleIndex)
        articleAudPos = self.Parent.channel.getAudioPosition(foundArticleIndex)
        articleUid = self.Parent.channel.getArticleUid(foundArticleIndex)
        if not articleUid:
          articleUid = str(uuid.uuid4())
        if inAllArticles:
          channelName = self.Parent.channel.getChannelNamefromArticle(foundArticleIndex)
          feedType = self.Parent.channel.getFeedTypefromArticle(foundArticleIndex)
        else:
          channelName = self.Parent.channel.getChannelName()
          feedType = self.Parent.channel.getFeedType()
        csvFileName = channelName + " (" + feedType + ")" + ".txt"
        archiveFile = os.path.join(archiveFolder, csvFileName)
        writeheader = False
        if not os.path.isfile(archiveFile):
          writeheader = True
        with open(archiveFile, 'a', encoding="utf8", newline='') as f:
          writer = csv.writer(f)
          if writeheader:
            # write the header
            writer.writerow(articlesHeader)
          # write article
          articleRow = [articleTitle, articleDesc, articleDate, articleLink, "0", articleAudio, articleAudPos, articleUid]
          writer.writerow(articleRow)
        articlesToArchiveIds.append(itemIdToArchive) # Uložíme ID pro smazání

    # delete archived articles from original location
    self.DeleteArticleItem(None, True)
    # self.deleteArticleItemsByIds(articlesToArchiveIds)
    # if not already present, insert "archiv" to end of channels list
    if self.Parent.channelsList.getItem(self.Parent.channelsList.GetItemCount()-1) != self.Parent.archiveTitle:
      self.Parent.channelsList.addItem(self.Parent.channelsList.GetItemCount(), self.Parent.archiveTitle)

  def deleteArticleItemsByIds(self, idsToDelete):
    self.articlesList.Freeze()
    indicesToDelete = []
    for index in range(self.articlesList.GetItemCount()):
      itemId = self.articlesList.GetItemIdFromIndex(index)
      if itemId in idsToDelete:
        indicesToDelete.append(index)

    for indexToDelete in reversed(indicesToDelete):
      itemIdToDelete = self.articlesList.GetItemIdFromIndex(indexToDelete)
      foundArticleIndex = self.getArticleIndexById(itemIdToDelete)
      if foundArticleIndex != -1:
        if not inAllArticles:
          if self.Parent.inArchive:
            try:
              del self.Parent.channel.articles[foundArticleIndex]
            except IndexError:
              pass
          else:
            delMark = self.Parent.channel.articles[foundArticleIndex].find(self.Parent.channel.applyNs("deleted", self.Parent.channel.ns))
            if delMark is not None:
              delMark.text = "1"
            self.needSave = True

        self.articlesList.DeleteItem(indexToDelete)
        if searchFilter:
          itemstr = self.articlesList.GetItemText(indexToDelete)
          if self.Parent.channel.getChannelName() in self.Parent.searchResults and itemstr in self.Parent.searchResults[self.Parent.channel.getChannelName()]:
            try:
              self.Parent.searchResults[self.Parent.channel.getChannelName()].remove(itemstr)
            except ValueError:
              pass
    self.articlesList.Thaw()
    self.articlesList.Refresh()
    if self.articlesList.GetItemCount() > 0:
      if indicesToDelete:
        lastDeletedIndex = min(indicesToDelete)
        if lastDeletedIndex < self.articlesList.GetItemCount():
          self.articlesList.Select(lastDeletedIndex)
        elif self.articlesList.GetItemCount() > 0:
          self.articlesList.Select(self.articlesList.GetItemCount() - 1)
        self.onarticlesListChange(None)
    else:
      self.noArticles()

  def sendArticleItem(self, evt):
    index = self.getSelectedArticleIndex()
    subject = self.Parent.channel.getArticleTitle(index)
    body = self.Parent.channel.getArticleDescription(index)
    body = body + "\n"
    body = body + "Zdroj: " + self.Parent.channel.getArticleLink(index)
    webbrowser.open('mailto:?subject=' + subject + '&body=' + body, new=1)

  def sayArticleDateItem(self, evt):
    selItemsNum = self.articlesList.GetSelectedItemCount()
    if selItemsNum == 0:
      sayMessage = "Není nic vybráno."
    elif selItemsNum > 1:
      sayMessage = ngettext("Selected %s message", "Selected %s messages", selItemsNum) % selItemsNum
    else:
      artDateTime = None
      sayMessage = ""
      index = self.getSelectedArticleIndex()
      articleDate = self.Parent.channel.getArticleDate(index)
      artDateTime = getRightDateFormat(articleDate)
      if artDateTime:
        locale.setlocale(locale.LC_TIME, "czech")
        datestr = datetime.datetime.strftime(artDateTime, "%d. %B %Y %H:%M:%S")
        sayMessage = datestr
      else:
        # Translators: can not get the date
        sayMessage = _("Unable to retrieve date.")
      if sayMessage:
        sayMessage = sayMessage + ", "
      # Translators: Announcement of message and number of messages (e.g. Message 5 of 300)
      sayMessage = sayMessage + _("Message %s of %s") % (str(self.articlesList.GetFirstSelected() + 1), str(self.articlesList.GetItemCount()))
      if inAllArticles:
        sayMessage = self.Parent.channel.getChannelNamefromArticle(index) + ", " + sayMessage
    ui.message(sayMessage)

  def sayArticleDescriptionItem(self, evt):
    index = self.getSelectedArticleIndex()
    articleDesc = self.Parent.channel.getArticleDescription(index)
    if articleDesc:
      ui.message(articleDesc)
    else:
      ui.message("Žádný popis.")
        
  def openArticleItem(self, evt):
    if openArticlesAsText:
      self.openArticleAsTextItem(None)
    else:
      self.openArticleInBrowserItem(None)

  def openArticleInBrowserItem(self, evt):
    index = self.getSelectedArticleIndex()
    articleAudioAddress = ""
    articleAddress = self.Parent.channel.getArticleLink(index)
    if articleAddress == "Nelze získat odkaz na zprávu.":
      articleAddress = ""
    # some podcasts have no article address, so try to obtain address for audio
    if not articleAddress:
      articleAudioAddress = self.Parent.channel.getAudioUrl(index)
    if articleAudioAddress == "Nelze získat odkaz na zprávu.":
      articleAudioAddress = ""
    if articleAddress:
      os.startfile(articleAddress)
    elif articleAudioAddress:
      os.startfile(articleAudioAddress)
    else:
      # Translators: Can not get the message address
      dlgMessage = _("Unable to retrieve the message address.")
      # Translators: Title of information dialog
      dlgTitle = _("Warning")
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)

  def displayText(self):
    self.Hide()
    self.Disable()
    try:
      TextWindow(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def playPodcast(self, address, title, position):
    if not vlcInstalled:
      # Translators: Message saying that title can not be played because VLC player was not found
      errorMessage = _("The title cannot be played because the VLC player was not found.")
      # Translators: Title of error dialog
      errorTitle = _("Warning")
      wx.MessageBox(errorMessage, errorTitle)
      return
    AudioPlayer.dlgTitle = title
    AudioPlayer.source = address
    if position:
      AudioPlayer.playingPos = float(position)
    else:
      AudioPlayer.playingPos = 0
    self.startPlaying()

  def startPlaying(self):
    self.Hide()
    self.Disable()
    try:
      AudioPlayer(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def openArticleAsTextItem(self, evt):
    index = self.getSelectedArticleIndex()
    # try to obtain MP3 link for podcast
    articleAddress = self.Parent.channel.getAudioUrl(index)
    if articleAddress:
      self.playPodcast(articleAddress, self.Parent.channel.getArticleTitle(index), self.Parent.channel.getAudioPosition(index))
      return
    # podcast MP3 link is empty, so get normal article link
    articleAddress = self.Parent.channel.getArticleLink(index)
    # some channels cannot be opened as text
    if "news.google.com" in articleAddress:
      # Translators: message Opening on the web
      ui.message(_("Opening the web browser."))
      self.openArticleInBrowserItem(None)
      return
    request_headers = {
      'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
    }
    if "idnes.cz" in articleAddress:
      # requires setting cookie
      req = requests.get(articleAddress, cookies={'dCMP': '"mafra=1111,all=1,reklama=1,part=0,cpex=1,google=1,gemius=1,id5=1,next=0000,onlajny=0000,jenzeny=0000,databazeknih=0000,autojournal=0000,skodahome=0000,skodaklasik=0000,groupm=1,piano=1,seznam=1,geozo=0,czaid=1,click=1,verze=2,'})
    elif "sciencemag.cz" in articleAddress:
      # cannot display without sending correct header
      req = requests.get(articleAddress, headers=request_headers)
    elif "letemsvetemapplem.eu" in articleAddress or "technologiebezzraku.sk" in articleAddress:
      # returns bad encoding, so correct utf-8 encoding must be set
      req = requests.get(articleAddress)
      req.encoding = req.apparent_encoding
    else:
      req = requests.get(articleAddress)
    pageHtml = req.text
    # find article starting point
    # search for h1
    headingPos = pageHtml.find("<h1")
    # if not found, search for <main
    if headingPos == -1:
      headingPos = pageHtml.find("<main")
    # when nothing is found, set starting pos to 0
    if headingPos == -1:
      headingPos = 0
    # find article end
    # search for </article> (from the end)
    endPos = pageHtml.rfind("</article>", headingPos)
    # if not found, search for next landmark
    if endPos == -1:
      endPos = pageHtml.find("<aside", headingPos)
    if endPos == -1:
      endPos = pageHtml.find("<nav", headingPos)
    if endPos == -1:
      endPos = pageHtml.find("<footer", headingPos)
    # if nothing is found, set end to end of text
    if endPos == -1:
      endPos = len(pageHtml)
    pageEditHtml = pageHtml[headingPos:endPos]

    soup = BeautifulSoup(pageEditHtml)
    pageText = soup.getText(separator="\n", strip=True)
    # Translators: Title of Text viewer window
    TextWindow.dlgTitle = _("Text viewer")
    TextWindow.dispText = pageText
    self.displayText()

  def saveArticles(self):
    global inAllArticles
    global deleteAllAfter
    if inAllArticles:
      # mark all deleted articles in all needed channels
      global articlesHeader
      dataError = False
      displayWarningDlg = False
      for channelName, articlesUids in deleteAllAfter.items():
        csvFileName = channelName + ".txt"
        articlesFile = os.path.join(articlesFolder, csvFileName)
        articlesFileOut = os.path.join(articlesFolder, csvFileName+"tmp")
        # open file with articles and temp file for storing copy with marked articles for deletion
        with open(articlesFile, 'r', encoding="utf8", newline='') as f, open(articlesFileOut, 'w', encoding="utf8", newline='') as fout:
          reader = csv.DictReader(f)
          writer = csv.writer(fout)
          # write the header
          writer.writerow(articlesHeader)
          for row in reader:
            try:
              if row['uid'] in articlesUids:
                row['deleted'] = "1"
            except:
              dataError = True
            try:
              rowout = [row['title'], row['description'], row['date'], row['link'], row['deleted'], row['audiourl'], row['audiopos'], row['uid']]
              writer.writerow(rowout)
            except:
              dataError = True
        # when data inconsistency appears, newly created tmp file will be erased and nothing changes in original file
        if os.path.isfile(articlesFileOut) and dataError:
          os.remove(articlesFileOut)
          dataError = False
          displayWarningDlg = True
        # if everything is ok, original file will be removed and tmp file will be renamed to original file
        if os.path.isfile(articlesFileOut) and not dataError:
          os.remove(articlesFile)
          os.rename(articlesFileOut, articlesFile)
      deleteAllAfter.clear()
      if displayWarningDlg:
        # Translators: something with data is wrong, it is necessary to make refresh by receiving messages
        displayWarningDlg = False
        dlgMessage = _("Some data inconsistency appeared. Please perform receiving messages operation by pressing F5 in feed list.")
        # Translators: Title of warning dialog
        dlgTitle = _("Warning")
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
    else:
      if self.Parent.inArchive:
        self.Parent.channel.writeArticles(True)
      else:
        self.Parent.channel.writeArticles()
    self.needSave = False

  def backtoChannelsItem(self, item):
    global inAllArticles
    if inAllArticles:
      if self.needSave:
        self.saveArticles()
      inAllArticles = False
    else:
      if self.needSave:
        self.saveArticles()
      # delete information about new messages
      selItem = self.Parent.channelsList.getSelectedItem()
      if selItem in self.Parent.newsChannels:
        del self.Parent.newsChannels[selItem]
        if self.Parent.filteredNews:
          if len(self.Parent.newsChannels) > 0:
            delItem = self.Parent.channelsList.GetFirstSelected()
            cursel = self.Parent.channelsList.GetFirstSelected()
            self.Parent.channelsList.deleteItem(delItem)
            if self.Parent.channelsList.GetItemCount() == cursel:
              cursel = cursel -1
            self.Parent.channelsList.selectItem(cursel)
          else:
            self.Parent.newsMenuItem.Check(False)
            self.Parent.displayChannelsFilterItem(None)
        self.Parent.onchannelsListChange(None)
      # when search results are displayed, refresh of additional info is needed
      if searchFilter:
        self.Parent.onchannelsListChange(None)
    self.Parent.Enable()
    self.Parent.Show()
    # when archive channel has no articles we have to delete it
    if self.Parent.inArchive:
      if self.Parent.channel.getNumberOfArticles() == 0:
        self.Parent.reallyDeleteChannel()
    self.GetParent().articlesWindow = None
    self.Destroy()

  def onarticlesListChange(self, evt):
    global currentArticleDescription
    global currentArticlePosition
    index = self.getSelectedArticleIndex()
    if index != -1:
      articleDesc = self.Parent.channel.getArticleDescription(index)
      if articleDesc:
        currentArticleDescription = articleDesc
      else:
        currentArticleDescription = ""
      articlePos = self.Parent.channel.getAudioPosition(index)
      if articlePos:
        articlePosnum = float(articlePos)
        percentPos = articlePosnum * 100
        # Translators: Text displayed in the message list when part of the podcast has been already played
        currentArticlePosition = _("played %s%%") % str(math.trunc(percentPos))
      else:
        currentArticlePosition = ""
    else:
      currentArticleDescription = ""

  def onarticlesListChoice(self, evt):
    self.openArticleItem(None)

  def onarticlesListKeyDown(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_ESCAPE:
      self.backtoChannelsItem(None)
    evt.Skip()

  def onClose(self, evt):
    global inAllArticles
    if inAllArticles:
      if self.needSave:
        self.saveArticles()
      inAllArticles = False
    else:
      if self.needSave:
        self.saveArticles()
    self.Parent.Destroy()
    self.Destroy()
    RssReader._instance = None


class ChannelPropertiesDialog(wx.Dialog):

  chName = ""
  chAddress = ""
  
  def __init__(self, parent):
    
    # The title of a dialog.
    super(ChannelPropertiesDialog, self).__init__(parent, title=_("Channel properties"))
    self.Bind(wx.EVT_CLOSE, self.onClose)

    # icon
    self.SetIcon(icon)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a channel name text field
    # Translators: the label of the channel name edit field in Channel properties dialog
    channelNameLabel = _("&Feed name:")
    self.channelNameEdit = sHelper.addLabeledControl(channelNameLabel, wx.TextCtrl)

    # The label of a channel address text field
    # Translators: the label of the channel address edit field in Channel properties dialog
    channelAddressLabel = _("Feed &address:")
    self.channelAddressEdit = sHelper.addLabeledControl(channelAddressLabel, wx.TextCtrl)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    # Translators: the label of the OK button in Channel properties dialog
    self.okButton = wx.Button(self, label=_("&OK"))
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    # Translators: the label of the Cancel button in Channel properties dialog
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_("&Cancel"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    # find channel name and address
    selItem = parent.channelsList.getSelectedItem()
    self.chName = ""
    self.chAddress = ""
    for outline in parent.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        self.chName = selItem
        self.chAddress = outline.get("xmlUrl")
        break
    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.channelNameEdit.SetValue(self.chName)
    self.channelAddressEdit.SetValue(self.chAddress)
    self.channelNameEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    if self.channelNameEdit == "" or self.channelAddressEdit == "":
      self.EndModal(False)
      self.onClose(self)
      return
    csvFileName = self.chName + ".txt"
    articlesFile = os.path.join(articlesFolder, csvFileName)
    # write changes to OPML file
    for outline in self.Parent.opmlTree.getroot().iter("outline"):
      if outline.get("title") == self.chName:
        self.chName = validFileName(self.channelNameEdit.Value)
        self.chAddress = self.channelAddressEdit.Value
        self.Parent.newChannelName = self.chName
        outline.set("title", self.chName)
        outline.set("text", self.chName)
        outline.set("xmlUrl", self.chAddress)
        outline.set("type", "rss")
        with open(opmlFilePath, 'wb') as f:
          f.write(ET.tostring(self.Parent.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # rename file with articles
    if os.path.isfile(articlesFile):
      newCsvFileName = self.chName + ".txt"
      newArticlesFile = os.path.join(articlesFolder, newCsvFileName)
      if articlesFile != newArticlesFile:
        os.rename(articlesFile, newArticlesFile)
    self.EndModal(True)
    self.onClose(self)

  def onCancel(self, evt):
    self.EndModal(False)
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class ChannelDatabaseDialog(wx.Dialog):
  
  passkey = False
  current_items = []

  def __init__(self, parent):
    # The title of a dialog
    # Translators: The title of channel database dialog
    super(ChannelDatabaseDialog, self).__init__(parent, title=_("RSS feed database"))

    # icon
    self.SetIcon(icon)

    self.current_path = []
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.Bind(wx.EVT_CLOSE, self.onClose)

    # get RSS categories from XML
    xmlUrl = "https://suscik.cz/nvda/rss/feeds2.xml"
    offlinedb = False
    try:
      response = urlopen(xmlUrl).read()
    except:
      offlinedb = True
    if not offlinedb:
      self.tree = ET.fromstring(response)
    else:
      xmlFile = os.path.join(os.path.dirname(__file__), "files", "feeds2.xml")
      treexml = ET.parse(xmlFile)
      self.tree = treexml.getroot()
      # Translators: The window title in case that locale database is used
      self.SetTitle(_("RSS feed database (local)"))
    self.current_items = self.get_items(self.tree)
      
    # listbox with categories
    self.categoriesList = wx.ListBox(self)
    self.update_listbox()
    self.categoriesList.Bind(wx.EVT_LISTBOX_DCLICK, self.on_enter)
    self.categoriesList.Bind(wx.EVT_KEY_UP, self.oncategoriesListKeyDown)
    sHelper.addItem(self.categoriesList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.categoriesList.SetFocus()
    self.CentreOnScreen()
    self.categoriesList.SetSelection(0)

  def get_items(self, element):
    items = []
    for child in element:
      if 'name' in child.attrib:
        items.append({"tag": child.tag, "name": child.attrib['name'], "element": child})
      elif child.tag == "Feed":
        items.append({"tag": child.tag, "description": child.get("Description"), "url": child.get("URL")})
    return items

  def update_listbox(self):
    self.categoriesList.Clear()
    sorted_items = sorted(self.current_items, key=lambda item: locale.strxfrm(item.get("description") or item.get("name")))
    display_items = []
    for item in sorted_items:
      if item["tag"] == "Feed":
        display_items.append((item["description"], item))
      else:
        display_items.append((item["name"], item))
    for display_text, _ in display_items:
      self.categoriesList.Append(display_text)
    self.current_items = [element for _, element in display_items]

  def on_enter(self, event=None):
    selection = self.categoriesList.GetSelection()
    if selection != wx.NOT_FOUND:
      selected_item = self.current_items[selection]
      if selected_item["tag"] == "Feed":
        self.Parent.dbSelectedUrl = selected_item['url']
        self.onClose(self)
      else:
        self.current_path.append((selected_item, selection))
        self.current_items = self.get_items(selected_item["element"])
        self.update_listbox()
        if self.categoriesList.GetCount() > 0:
          self.categoriesList.SetSelection(0)

  def on_back(self):
    if self.current_path:
      parent_item, previous_selection = self.current_path.pop()
      if self.current_path:
        self.current_items = self.get_items(self.current_path[-1][0]["element"])
      else:
        self.current_items = self.get_items(self.tree)
      self.update_listbox()
      if self.categoriesList.GetCount() > previous_selection:
        self.categoriesList.SetSelection(previous_selection)
      else:
        self.categoriesList.SetSelection(self.listbox.GetCount() - 1)

  def oncategoriesListKeyDown(self, evt):
    key = evt.GetKeyCode()
    if self.passkey:
      self.passkey = False
      return
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      self.on_enter()
    elif key == wx.WXK_BACK:
      self.on_back()
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()

class PodcastSearchDialog(wx.Dialog):
  
  passkey = False

  def __init__(self, parent, podcasts):
    # The title of a dialog
    # Translators: The title of podcast search dialog
    super(PodcastSearchDialog, self).__init__(parent, size=(1000, 800), title=_("Found podcasts"))

    self.passkey = True
    # icon
    self.SetIcon(icon)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.Bind(wx.EVT_CLOSE, self.onClose)

    # filter podcast to items with feed, sorted with local preferences
    self.validPodcasts = sorted([p for p in podcasts if p.feed], key=lambda p: locale.strxfrm(p.name))
    # Translators: The title of search podcasts dialog with number of found podcasts
    self.SetTitle(_("Found podcasts: %s") % str(len(self.validPodcasts)))

    # prepare items for listbox
    self.choices = []
    for item in self.validPodcasts:
      resstr = item.name
      if item.category:
        # Translators: word Category in the list of found podcasts
        resstr = resstr + _(". Category: ") + item.category
      if item.episode_count:
        # Translators: Number of episodes in the list of found podcasts
        resstr = resstr + _(". Number of episodes: ") + str(item.episode_count)
      if item.author:
        # Translators: word Author in the list of found podcasts
        resstr = resstr + _(". Author: ") + item.author + "."
      self.choices.append(resstr)

    # listbox with search results
    self.podcastsList = wx.ListBox(self, choices=self.choices)
    self.podcastsList.Bind(wx.EVT_LISTBOX_DCLICK, self.onEnter)
    self.podcastsList.Bind(wx.EVT_KEY_UP, self.onpodcastsListKeyUp)
    sHelper.addItem(self.podcastsList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.podcastsList.SetFocus()
    self.CentreOnScreen()
    self.podcastsList.SetSelection(0)

  def onEnter(self, event=None):
    selection = self.podcastsList.GetSelection()
    if selection != wx.NOT_FOUND:
      selectedPodcast = self.validPodcasts[selection]
      self.Parent.dbSelectedUrl = selectedPodcast.feed
      self.onClose(self)

  def onpodcastsListKeyUp(self, evt):
    key = evt.GetKeyCode()
    if self.passkey:
      self.passkey = False
      return
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      self.onEnter()
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()


class ChannelSettingsDialog(wx.Dialog):

  channel = None

  def __init__(self, parent):
    
    # The title of a dialog.
    # Translators: Title of dialog Channel setting
    super(ChannelSettingsDialog, self).__init__(parent, title=_("Settings for feed %s") % self.channel.getChannelName())

    # icon
    self.SetIcon(icon)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.channel.loadSettings()
    
    # The save audio files checkbox
    # Translators: checkbox Save audio files in Channel settings dialog
    self.channelSaveAudioCheck = wx.CheckBox(self, -1, _("&Save audio files"))
    self.channelSaveAudioCheck.SetValue(bool(self.channel.saveAudioSetting))
    sHelper.addItem(self.channelSaveAudioCheck)

    # the prefix podcast file combobox
    # Translators: The combo box audio filename prefix choices in feed settings dialog
    prefixChoices = [_("Not set"), _("Nothing"), _("Date and time"), _("Sequence number")]
    # Translators: Combobox Add before name of audio file in Channel settings dialog
    self.channelPrefixPodcastFileCombo = sHelper.addLabeledControl(_("Add before the podcast filename"), wx.Choice, choices=prefixChoices)
    self.channelPrefixPodcastFileCombo.SetSelection(self.channel.prefixPodcastFileSetting)

    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    # Translators: button OK in Channel settings dialog
    self.okButton = wx.Button(self, label=_("&OK"))
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    # Translators: Button Cancel in Channel settings dialog
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_("&Cancel"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.channelSaveAudioCheck.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    self.channel.saveAudioSetting = self.channelSaveAudioCheck.GetValue()
    self.channel.prefixPodcastFileSetting = self.channelPrefixPodcastFileCombo.GetSelection()
    self.channel.saveSettings()
    self.onClose(None)

  def onCancel(self, evt):
    self.onClose(None)

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()


class RssReaderSettingsDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    # Translators: Title of dialog RSS reader settings
    super(RssReaderSettingsDialog, self).__init__(parent, title=_("Settings for RSS Reader"))

    # icon
    self.SetIcon(icon)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The open articles as text checkbox
    # Translators: checkbox Open articles as text or audio in RSS reader settings dialog
    self.openArticlesAsTextCheck = wx.CheckBox(self, -1, _("Open messages as &text or audio"))
    self.openArticlesAsTextCheck.SetValue(bool(openArticlesAsText))
    sHelper.addItem(self.openArticlesAsTextCheck)

    # The ask article deletion checkbox
    # Translators: checkbox ask before message deletion in RSS reader settings dialog
    self.askArticleDeletionCheck = wx.CheckBox(self, -1, _("Ask before message &deletion"))
    self.askArticleDeletionCheck.SetValue(bool(askArticleDeletion))
    sHelper.addItem(self.askArticleDeletionCheck)

    # The ask article archivation checkbox
    # Translators: checkbox Ask before moving message to archive in RSS reader settings dialog
    self.askArticleArchivationCheck = wx.CheckBox(self, -1, _("Ask before removing message to &archive"))
    self.askArticleArchivationCheck.SetValue(bool(askArticleArchivation))
    sHelper.addItem(self.askArticleArchivationCheck)

    # The automatically announce number of new messages checkbox
    # Translators: checkbox Automatically announce number of channel new messages in RSS reader settings dialog
    self.announceNumberofNewsCheck = wx.CheckBox(self, -1, _("Announce the number of &messages in selected feed	F2"))
    self.announceNumberofNewsCheck.SetValue(bool(automaticallyAnnounceNumberofNews))
    sHelper.addItem(self.announceNumberofNewsCheck)

    # The automatically announce description checkbox
    # Translators: checkbox Automatically announce message description in RSS reader settings dialog
    self.announceDescriptionCheck = wx.CheckBox(self, -1, _("Automatically announce message &description"))
    self.announceDescriptionCheck.SetValue(bool(automaticallyAnnounceDescription))
    sHelper.addItem(self.announceDescriptionCheck)

    # The new articles after opening checkbox
    # Translators: checkbox Load new messages after opening the channel in RSS reader settings dialog
    self.newArticlesAfterOpeningCheck = wx.CheckBox(self, -1, _("Load new messages after opening &feed"))
    self.newArticlesAfterOpeningCheck.SetValue(bool(newArticlesAfterOpening))
    sHelper.addItem(self.newArticlesAfterOpeningCheck)

    # The download articles after startup checkbox
    # Translators: checkbox Load new messages after startup in RSS reader settings dialog
    self.downloadArticlesAfterStartupCheck = wx.CheckBox(self, -1, _("Receive new messages after startup"))
    self.downloadArticlesAfterStartupCheck.SetValue(bool(downloadArticlesAfterStartup))
    sHelper.addItem(self.downloadArticlesAfterStartupCheck)

    # The download articles interval combobox
    # Translators:  Interval combo box choices
    intervalChoices = [_("Off"), _("5 minutes"), _("10 minutes"), _("15 minutes"), _("30 minutes"), _("1 hour"), _("2 hours"), _("3 hours")]
    # Translators: Combobox Automatically download new messages in RSS reader settings dialog
    self.downloadArticlesIntervalCombo = sHelper.addLabeledControl(_("Automatically &receive new messages"), wx.Choice, choices=intervalChoices)
    self.downloadArticlesIntervalCombo.Bind(wx.EVT_CHOICE, self.ondownloadArticlesIntervalComboChoice)
    self.downloadArticlesIntervalCombo.SetSelection(downloadArticlesInterval)

    # the go to tray after NVDA starts checkbox
    # Translators: checkbox Automatically to tray after NVDA startup in RSS reader settings dialog
    self.toTrayOnNvdaStartCheck = wx.CheckBox(self, -1, _("To &system tray after startup"))
    self.toTrayOnNvdaStartCheck.SetValue(bool(toTrayOnNvdaStart))
    if self.downloadArticlesIntervalCombo.GetSelection() == 0:
      self.toTrayOnNvdaStartCheck.Hide()
    else:
      self.toTrayOnNvdaStartCheck.Show()
    sHelper.addItem(self.toTrayOnNvdaStartCheck)

    # The delete older articles combobox
    # Translators: The delete older articles combo box choices
    deleteChoices = [_("Off"), _("One day"), _("Two days"), _("Three days"), _("Five days"), _("One week"), _("Two weeks"), _("Three weeks"), _("One month")]
    # Translators: Combobox Automatically delete old messages in RSS reader settings dialog
    self.deleteOlderArticlesCombo = sHelper.addLabeledControl(_("Automatically delete &older messages"), wx.Choice, choices=deleteChoices)
    self.deleteOlderArticlesCombo.SetSelection(deleteOlderArticles)

    # The save audio from podcasts checkbox
    # Translators: checkbox Save audio podcast files in RSS reader settings dialog
    self.saveAudioFromPodcastsCheck = wx.CheckBox(self, -1, _("Save podcasts audio &files"))
    self.saveAudioFromPodcastsCheck.SetValue(bool(saveAudioFromPodcasts))
    sHelper.addItem(self.saveAudioFromPodcastsCheck)

    # The podcasts mp3 path editbox
    # Translators: Edit field Podcast folder path in RSS reader settings dialog
    self.podcastsMp3PathEdit = sHelper.addLabeledControl(_("Audio files folder"), wx.TextCtrl)
    self.podcastsMp3PathEdit.SetValue(podcastsMp3Path)

    # the select folder button
    # Translators: button for selecting folder in RSS reader settings dialog
    self.selectFolderButton = wx.Button(self, label=_("&Open folder with stored audio files..."))
    self.selectFolderButton.Bind(wx.EVT_BUTTON, self.onSelectFolder)
    sHelper.addItem(self.selectFolderButton)

    # the prefix podcast file combobox
    # Translators: The combo box audio filename prefix choices in settings dialog
    prefixChoices = [_("Nothing"), _("Date and time"), _("Sequence number")]
    # Translators: combobox Add before filename in RSS reader settings dialog
    self.prefixPodcastFileCombo = sHelper.addLabeledControl(_("Add before the podcast filename"), wx.Choice, choices=prefixChoices)
    self.prefixPodcastFileCombo.SetSelection(prefixPodcastFile)

    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    # Translators: button OK in RSS reader settings dialog
    self.okButton = wx.Button(self, label=_("&OK"))
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    # Translators: button Cancel in RSS reader settings dialog
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_("&Cancel"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.openArticlesAsTextCheck.SetFocus()
    self.CentreOnScreen()

  def onSelectFolder(self, evt):
    global podcastsMp3Path
    if podcastsMp3Path:
      initialFolder = podcastsMp3Path
    else:
      initialFolder = ""
    # Translators: title if dialog for selecting file podcasts folder
    selectFolderDialog = wx.DirDialog(self, _("Select a folder to store audio files"), defaultPath=initialFolder)
    if selectFolderDialog.ShowModal() == wx.ID_OK:
      self.podcastsMp3PathEdit.SetValue(selectFolderDialog.GetPath())

  def ondownloadArticlesIntervalComboChoice(self, evt):
    sel = self.downloadArticlesIntervalCombo.GetSelection()
    if sel == 0:
      self.toTrayOnNvdaStartCheck.Hide()
    else:
      self.toTrayOnNvdaStartCheck.Show()
      parent = self.GetParent()
      if parent:
        parent.Layout()

  def onOk(self, evt):
    global defaultPodcastMp3Folder
    # when podcastsMp3Path does not exist, assign and create default one
    if not os.path.exists(self.podcastsMp3PathEdit.GetValue()):
      if self.podcastsMp3PathEdit.GetValue() != defaultPodcastMp3Folder:
        # Translators: message that set folder cannot be found, setting default folder
        dlgMessage = _("The %s folder could not be found, setting the default folder.") % self.podcastsMp3PathEdit.GetValue()
        # Translators: Title of warning dialog
        dlgTitle = _("Warning")
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        self.podcastsMp3PathEdit.SetValue(defaultPodcastMp3Folder)
      if not os.path.exists(self.podcastsMp3PathEdit.GetValue()):
        os.mkdir(self.podcastsMp3PathEdit.GetValue())
    # Load settings
    SetFile = settingsFilePath
    with open(SetFile, 'r') as f:
      config = json.load(f)
    # Assign values to config
    config['OpenArticlesAsText'] = self.openArticlesAsTextCheck.GetValue()
    config['AskArticleDeletion'] = self.askArticleDeletionCheck.GetValue()
    config['AskArticleArchivation'] = self.askArticleArchivationCheck.GetValue()
    config['AutomaticallyAnnounceNumberofNews'] = self.announceNumberofNewsCheck.GetValue()
    config['AutomaticallyAnnounceDescription'] = self.announceDescriptionCheck.GetValue()
    config['NewArticlesAfterOpening'] = self.newArticlesAfterOpeningCheck.GetValue()
    config['DownloadArticlesAfterStartup'] = self.downloadArticlesAfterStartupCheck.GetValue()
    config['DownloadArticlesInterval'] = self.downloadArticlesIntervalCombo.GetSelection()
    config['ToTrayOnNvdaStart'] = self.toTrayOnNvdaStartCheck.GetValue()
    config['DeleteOlderArticles'] = self.deleteOlderArticlesCombo.GetSelection()
    config['PodcastsMp3Path'] = self.podcastsMp3PathEdit.GetValue()
    config['SaveAudioFromPodcasts'] = self.saveAudioFromPodcastsCheck.GetValue()
    config['PrefixPodcastFile'] = self.prefixPodcastFileCombo.GetSelection()
    # Save the settings
    with open(SetFile, 'w') as f:
      json.dump(config, f)
    # Apply settings
    global openArticlesAsText
    openArticlesAsText = config['OpenArticlesAsText']
    global askArticleDeletion
    askArticleDeletion = config['AskArticleDeletion']
    global askArticleArchivation
    askArticleArchivation = config['AskArticleArchivation']
    global automaticallyAnnounceNumberofNews 
    automaticallyAnnounceNumberofNews = config['AutomaticallyAnnounceNumberofNews']
    global automaticallyAnnounceDescription 
    automaticallyAnnounceDescription = config['AutomaticallyAnnounceDescription']
    global newArticlesAfterOpening
    newArticlesAfterOpening = config['NewArticlesAfterOpening']
    global downloadArticlesAfterStartup
    downloadArticlesAfterStartup = config['DownloadArticlesAfterStartup']
    global downloadArticlesInterval
    downloadArticlesInterval = config['DownloadArticlesInterval']
    global toTrayOnNvdaStart
    toTrayOnNvdaStart = config['ToTrayOnNvdaStart']
    global deleteOlderArticles
    deleteOlderArticles = config['DeleteOlderArticles']
    global podcastsMp3Path
    podcastsMp3Path = config['PodcastsMp3Path']
    global saveAudioFromPodcasts
    saveAudioFromPodcasts = config['SaveAudioFromPodcasts']
    global prefixPodcastFile
    prefixPodcastFile = config['PrefixPodcastFile']
    if downloadArticlesInterval == 0:
      self.Parent.regularTimer.Stop()
    else:
      self.Parent.regularTimer.Start(convertDownloadArticlesInterval(downloadArticlesInterval))
    self.onClose(None)

  def onCancel(self, evt):
    self.onClose(None)

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

  # category for addon commands in input gestures dialog
  scriptCategory = _addonSummary

  def __init__(self, *args, **kwargs):
    super(GlobalPlugin, self).__init__(*args, **kwargs)

    self.toolsMenu = gui.mainFrame.sysTrayIcon.toolsMenu
    self.runRssItem = self.toolsMenu.Append(wx.ID_ANY, mainTitle)
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onRssReader, self.runRssItem)

    # check for update
    self.updateCheck()

    # load config
    readcfg()

    # when set and download interval is also set, start app minimized
    if toTrayOnNvdaStart and downloadArticlesInterval > 0:
      self.onRssReader(None, True)

  def updateCheck(self):
    isNewVersion = False
    #determines the version from the web
    data = ""
    versionUrl = "https://suscik.cz/nvda/rss/ver.txt"
    try:
      response = urlopen(versionUrl).read()
    except:
      response = ""
    if response:
      data = response.decode()
    if data:
      #checks if new version is available
      isNewVersion = False
      currentVersion = data
      if (int(currentVersion.split(".")[0]) > int(_addonVersion.split(".")[0])):
        isNewVersion = True
      elif (int(currentVersion.split(".")[1]) > int(_addonVersion.split(".")[1])):
        isNewVersion = True
      elif (int(currentVersion.split(".")[2]) > int(_addonVersion.split(".")[2])):
        isNewVersion = True
    if isNewVersion:
      wx.CallAfter(self.updateAddon, currentVersion)
  
  def updateAddon(self, version):
    message = '%s %s %s %s.\n%s' % (
# Translators: message that new update is available
      _("An update is available"), _addonSummary, "verze", version,
      # Translators: Question whether user wants to download new app version
      _("Do you want to download it?"))
    # Translators: Title of checking for update window
    if gui.messageBox(message, _("Check for update"), wx.YES | wx.NO | wx.ICON_QUESTION) == wx.YES:
      #save to downloads
      updateUrl = "https://suscik.cz/nvda/rss/rssreader.nvda-addon"
      target = os.path.join(expanduser("~"), "downloads", "rssreader.nvda-addon")
      downloader = Downloader(downloadUrl=updateUrl, downloadTarget=target, callingProcess="addon update")
      downloader.startDownload()

  def chooseNVDAObjectOverlayClasses(self, obj, clsList):
    try:
      winClsName = obj.windowClassName
    except:
      winClsName = None
    if not winClsName:
      return
    if (
      obj.windowClassName == "SysListView32"
      and api.getFocusObject().appModule.appName == "nvda"
    ):
      ancestors = api.getFocusAncestors()
      for anObj in ancestors:
        if (anObj.windowHandle == channelHwnd or anObj.windowHandle == articleHwnd):
          clsList.insert(0, ClsRssList)
          break

  def onRssReader(self, evt, runminimized=False):
    gui.mainFrame.prePopup()
    d = RssReader(gui.mainFrame, runminimized)
    if runminimized:
      d.taskbarIcon.SetIcon(icon, mainTitle)
      d.Hide()
    else:
      d.Show()
    gui.mainFrame.postPopup()

  @script(
    # Translators: Gesture description for the Input gestures settings dialog
    description=_("&Show RSS reader window"),
    gesture="kb:control+shift+i"
  )
  def script_displayRss(self, gesture):
    wx.CallAfter(self.onRssReader, None)


class ClsRssList(IAccessible):

  def reportFocus(self):
    messageList = []
    obj = api.getForegroundObject()
    title = obj.name
    if self.name:
      messageList.append(self.name)
      if title.startswith(mainTitle):
        if searchFilter:
          if currentChannelResults:
            # Translators: string Found messages which is appended to channel names in case that the search text is active
            messageList.append(_("Found messages ") + currentChannelResults)
        elif automaticallyAnnounceNumberofNews:
          if currentChannelNewMessages:
            # Translators: string New messages which is appended to channels if automatic announcement of new messages in channels is set
            messageList.append(_("New messages %s") % currentChannelNewMessages)
      # Translators: first word of messages window title
      elif title.startswith(_("Messages")):
        messageList.append(currentArticlePosition)
        if automaticallyAnnounceDescription:
          messageList.append(currentArticleDescription)
      ui.message(". ".join(messageList))
    else:
      super(ClsRssList,self).reportFocus()

  def event_stateChange(self):
    # only to force NVDA not saying "selected" after each list item
    pass
